import 'package:intl/intl.dart';

class DateTimeFormat {
  DateFormat dateFormat;

  DateTimeFormat(String format) {
    dateFormat = new DateFormat(format);
  }

  String format(DateTime date) {
    return dateFormat?.format(date);
  }

  DateTime parse(String inputString) {
    return dateFormat?.parse(inputString);
  }
}

///时间格式
class DateTools {
  ///获取当前时间
  static DateTime now() {
    return new DateTime.now();
  }

  ///获取当前Unix时间(毫秒)
  static int unixMilliseconds() {
    return new DateTime.now().millisecondsSinceEpoch;
  }

  ///格式化时间字符串
  ///format: yyyy-MM-dd HH:mm:ss
  static String formatDateTime(DateTime date, String format) {
    DateTimeFormat dateFormat = new DateTimeFormat(format);
    return dateFormat.format(date);
  }

  ///分析时间字符串
  static DateTime parseDateTime(String inputString, String format) {
    DateTimeFormat dateFormat = new DateTimeFormat(format);
    return dateFormat.parse(inputString);
  }

  ///格式化成多久之前
  static String diffDateFromNow(DateTime datetime) {
    if (datetime != null) {
      Duration duration = new DateTime.now().difference(datetime);
      if (duration.inDays > 0) {
        return "${duration.inDays}天前";
      } else if (duration.inHours > 0) {
        return "${duration.inHours}小时前";
      } else if (duration.inMinutes > 0) {
        return "${duration.inMinutes}分钟前";
      } else {
        return "刚刚";
      }
    }
    return null;
  }

  static final int secondsADay = 86400;
  static final int milisecondADay = 86400*1000;

  ///是否同一天
  static bool IsSameDay(DateTime oneDay, DateTime anotherDay) {
    if (oneDay != null && anotherDay != null) {
      return oneDay.year == anotherDay.year &&
          oneDay.month == anotherDay.month &&
          oneDay.day == anotherDay.day; //same day
    } else {
      return false;
    }
  }

  ///是否24小时内
  static bool within24Hours(DateTime date) {
    return date != null && new DateTime.now()
        .difference(date)
        .inDays < 1;
  }
}
