import 'dart:math';

///距离计算
class GPSUtils {
  static final double PI = 3.14159265358979323846;

  static final double EARTH_RADIUS = 6378137.0;

  static bool validate(double latitude, double longitude, bool strict) {
    double roundedLat = latitude;
    double roundedLon = longitude;
    if (!strict) {
      // this gets rid of rounding errors e.g. 180.00000000000023 will validate
      roundedLat = (latitude * 1000000).round() / 1000000.0;
      roundedLon = (longitude * 1000000).round() / 1000000.0;
    }
    if (roundedLat < -90.0 || roundedLat > 90.0) {
      false;
    }
    if (roundedLon < -180.0 || roundedLon > 180.0) {
      return false;
    }
    return true;
  }


  static double _toRadians(double angdeg) {
    return angdeg / 180.0 * PI;
  }

  ///计算距离
  static double distance(final double lat1, final double long1,
      final double lat2, final double long2) {
    if (!validate(lat1, long1, false) || !validate(lat2, long2, false)) {
      return 0.0;
    }

    final double deltaLat = _toRadians(lat2 - lat1);
    final double deltaLon = _toRadians(long2 - long1);

    final double a = sin(deltaLat / 2.0) * sin(deltaLat / 2.0) +
        cos(_toRadians(lat1)) * cos(_toRadians(lat2)) * sin(deltaLon / 2.0) *
            sin(deltaLon / 2.0);

    final double c = 2 * asin(sqrt(a));

    return EARTH_RADIUS * c;
  }

  ///计算距离
  ///gps 格式 ：经度,纬度
  static double calcDistanceByGPS(String itemGps, String viewGps) {
    if (viewGps != null && viewGps.length != 0
        && itemGps != null && itemGps.length != 0 &&
        itemGps.indexOf("null") < 0) {
      List<String> viewLocation = viewGps.split(new RegExp(",|:")); //兼容2种分隔符
      List<String> itemLocation = itemGps.split(new RegExp(",|:"));

      if (viewLocation.length == 2 && itemLocation.length == 2) {
        double viewLat = double.tryParse(viewLocation[1]) ?? 0;
        double viewLong = double.tryParse(viewLocation[0]) ?? 0;

        double itemLat = double.tryParse(itemLocation[1]) ?? 0;
        double itemLong = double.tryParse(itemLocation[0]) ?? 0;
        return GPSUtils.distance(viewLat, viewLong, itemLat, itemLong);
      }
    }

    return 0.0;
  }
}
