import 'dart:convert';

class StringUtils {
  static String replaceSpecial(String str, String special, int start, int end) {
    if (str == null) {
      return null;
    } else {
      int len;
      if (start > end) {
        len = start;
        start = end;
        end = len;
      }

      len = str.length;
      if (start < 0) {
        start = 0;
      }

      if (start > len) {
        start = len;
      }

      if (end < 0) {
        end = 0;
      }

      if (end > len) {
        end = len;
      }

      if (isBlank(special)) {
        special = "*";
      }

      String specialStr = "";

      for (int i = start; i < end; ++i) {
        specialStr = specialStr + special;
      }

      return overlay(str, specialStr, start, end);
    }
  }

  static String replaceSpecialForNickName(String str, String special, int start,
      int end) {
    if (str == null) {
      return null;
    } else {
      int len;
      if (start > end) {
        len = start;
        start = end;
        end = len;
      }

      len = str.length;
      if (start < 0) {
        start = 0;
      }

      if (start > len) {
        start = len;
      }

      if (end < 0) {
        end = 0;
      }

      if (end > len) {
        end = len;
      }

      if (isBlank(special)) {
        special = "*";
      }

      return overlay(str, special, start, end);
    }
  }

  static bool isBlank(String str) {
    String tmpStr = str;
    if (str != null && str.length != 0) {
      return tmpStr.trim().length == 0;
    } else {
      return true;
    }
  }

  static bool isNotBlank(String str) {
    String tmpStr = str;
    if (str != null && str.length != 0) {
      return tmpStr.trim().length > 0;
    } else {
      return false;
    }
  }

  static String overlay(String str, String overlay, int start, int end) {
    if (str == null) {
      return null;
    } else {
      if (overlay == null) {
        overlay = "";
      }

      int len = str.length;
      if (start < 0) {
        start = 0;
      }

      if (start > len) {
        start = len;
      }

      if (end < 0) {
        end = 0;
      }

      if (end > len) {
        end = len;
      }

      if (start > end) {
        int temp = start;
        start = end;
        end = temp;
      }

      return str.substring(0, start) + overlay + str.substring(end);
    }
  }

  static bool isEmpty(String str) {
    return str == null || str.length == 0;
  }

  // URL decoder decodes url encoded utf-8 bytes
// Use this method to decode query string
// We need this kind of encoder and decoder with optional [encType] argument
  static String urlDecode(String s) {
    if(StringUtils.isBlank(s)){
      return "";
    }
    int i, p, q;
    var ol = new List<int>();
    for (i = 0; i < s.length; i++) {
      if (s[i].codeUnitAt(0) == 0x2b) ol.add(0x20); // convert + to space
      else
      if (s[i].codeUnitAt(0) == 0x25) { // convert hex bytes to a single bite
        i++;
        p = s[i].toUpperCase().codeUnitAt(0) - 0x30;
        if (p > 9) p = p - 7;
        i++;
        q = s[i].toUpperCase().codeUnitAt(0) - 0x30;
        if (q > 9) q = q - 7;
        ol.add(p * 16 + q);
      }
      else
        ol.add(s[i].codeUnitAt(0));
    }
    return utf8.decode(ol);
  }

// URL encoder encodes string into url encoded utf-8 bytes
// Use this method to encode cookie string
// or to write URL encoded byte data into OutputStream
  //能够将+#等特殊字符编码,空格转为%20
  static String urlEncode(String s) {
    if(StringUtils.isBlank(s)){
      return "";
    }
    int i, p, q;
    var ol = new List<int>();
    Utf8Encoder utf8encoder = new Utf8Encoder();
    List<int> il = utf8encoder.convert(s); //utf.encodeUtf8(s);

    for (i = 0; i < il.length; i++) {
      if (il[i] == 0x20) ol.add(0x2b); // convert sp to +
      else
      if (il[i] == 0x2a || il[i] == 0x2d || il[i] == 0x2e || il[i] == 0x5f) ol
          .add(il[i]); // do not convert
      else if (((il[i] >= 0x30) && (il[i] <= 0x39)) ||
          ((il[i] >= 0x41) && (il[i] <= 0x5a)) ||
          ((il[i] >= 0x61) && (il[i] <= 0x7a)))
        ol.add(il[i]);
      else { // '%' shift
        ol.add(0x25);
        ol.add((il[i] ~/ 0x10).toRadixString(16).codeUnitAt(0));
        ol.add((il[i] & 0xf).toRadixString(16).codeUnitAt(0));
      }
    }
    return new String.fromCharCodes(ol).replaceAll("+", "%20");
  }

  //dart 自带urlEncode跟Java相差太大。都是用上面两个API
//  //能够将空格编码为%20， 但是对+#等特殊字符不编码，
  static String urlEncodeOnlyParams(String url){
    if(StringUtils.isBlank(url)){
      return "";
    }
    return Uri.encodeFull(url);
  }
  //建议使用上面的API，这两个API后续会废弃
  static String urlDecodeOnlyParams(String url){
    if(StringUtils.isBlank(url)){
      return "";
    }
    return Uri.decodeFull(url);
  }
}

//
//void main(){
//  var uri = 'http://baidu.com/sfs?foo=some message测试';
//  var encoded = StringUtils.urlEncodeOnlyParams(uri);
//  print(encoded);
//
//  var decoded = StringUtils.urlDecodeOnlyParams(encoded);
//  print(decoded);
//print("---------------------------");
//  String encodedString = StringUtils.urlEncode(uri);
//  print("URL encoded string : $encodedString");
//  String decodedString = StringUtils.urlDecode("http%3A%2F%2Fbaidu.com%2Fsfs%20fe%E6%B5%8B%E8%AF%95");
//  print("URL decoded string : $decodedString");
//}