import 'package:saker/framework/services/service_base.dart';
import 'dart:math';

///版本比较
class VersionUtil {
  static final String JAXWS_VERSION_20 = "2.0";
  static final String JAXWS_VERSION_DEFAULT = "2.0";
  static final String PLATFORM_ANDROID = "android";
  static final String PLATFORM_IOS = "iphone";

  static bool isVerifyNeed(String ios, String android, MtopContext context) {
    String platform = context.platform;
    String appVersion = context.appVersion;

    if (ios != null &&
        platform != null &&
        platform == PLATFORM_IOS &&
        compareVersion(appVersion, ios) > 0) {
      return true;
    } else if (android != null &&
        platform != null &&
        platform == PLATFORM_ANDROID &&
        compareVersion(appVersion, android) > 0) {
      return true;
    }
    return false;
  }

  static int compareVersion(String version1, String version2) {
    if (version1 == null && version2 == null) {
      return 0;
    }
    if (version1 == null) {
      return -1;
    }
    if (version2 == null) {
      return 1;
    }
    List<String> versionArray1 = version1.split("\.");
    List<String> versionArray2 = version2.split("\.");
    int idx = 0;
    int minLength = min(versionArray1.length, versionArray2.length); // 取最小长度值
    int diff = 0;
    while (idx < minLength &&
        (diff = versionArray1[idx].length - versionArray2[idx].length) == 0 // 先比较长度
        &&
        (diff = int.tryParse(versionArray1[idx]).compareTo(int.tryParse(versionArray2[idx]))) == 0) {
      // 再比较大小
      ++idx;
    }
    // 如果已经分出大小，则直接返回，如果未分出大小，则再比较位数，有子版本的为大；
    diff = (diff != 0) ? diff : versionArray1.length - versionArray2.length;

    return diff;
  }

  /**
   * 兼容老版本对5.9.2版本及以后的单视频宝贝的显示问题
   *
   * @return
   */
  static bool isHigher6_2_6(MtopContext context) {
    return compareVersion(context.appVersion, "6.2.6") > 0;
  }

  static bool isHigherThan(String currVersion, String compareToVersion) {
    return compareVersion(currVersion, compareToVersion) > 0;
  }

  static bool isHigherAndEqual(String currVersion, String compareToVersion) {
    return compareVersion(currVersion, compareToVersion) >= 0;
  }

  static bool isLowerThan(String currVersion, String compareToVersion) {
    return compareVersion(currVersion, compareToVersion) < 0;
  }

  static bool isLowerAndEqual(String currVersion, String compareToVersion) {
    return compareVersion(currVersion, compareToVersion) <= 0;
  }
}

//void main(){
//  String currVersion = "6.2.6";
//  String compreToVersion = "6.2.6";
//  print(VersionUtil.isHigherThan(currVersion,compreToVersion));
//  print(VersionUtil.isHigherAndEqual(currVersion,compreToVersion));
//  print(VersionUtil.isLowerThan(currVersion,compreToVersion));
//  print(VersionUtil.isLowerAndEqual(currVersion,compreToVersion));
//}
