#Baas模块使用说明：
    Baas即后端即服务，利用IaaS , PaaS ,API以及SDK基础， 把后端能力以服务的形式提供出来。
    本Baas模块主要提供数据存储功能，根据数据分类，将接口分为三种类型：结构化数据接口，半结构化数据接口，和非结构化数据接口
#一：Baas模块初始化
    需要使用者在提供在Baas服务管控页面申请到的验证信息，不同Baas服务提供商给出的验证信息不同，开发同学以{key:value}的形式传给初始化接口即可。
    以阿里云Baas为例，阿里云Baas提供了AppId,eventid, masterkey三个值，因此初始化的时候，只需要将三个值和对应的key传给初始化函数，
        //阿里云Baas提供的三个验证值
        Map<String,String> baasConfig = {
            "appID":"TEm0YQBvXKPoWEFA",
            "envId":"1vqRR89Qssensuvj",
            "masterKey":"dmaj6yWHfnqM9Xaw7ZPTan9EktEUikdh0wuu"
        };
        //初始化
        MBaasSDK mBaasSDK = MBaasSDK.getMBaasSDK(baasConfig,baasProvider: BaasProvider.aliyun);
      
#二：结构化数据接口
    本Baas模块提供了如下几个结构化数据（二维表结构来逻辑表达和实现的数据，业务数据字段在业务开始就确定下来）操作接口：
   ###1. DB初始化接口：
        DataBase getDB(String dbName);
        获取DB接口操作对象，然后通过该DB对象操作数据接口。
        参数是DB名
   ####示例：     
        if (mBaasSDK != null) {
          baasDB = mBaasSDK.getDB('GameScore');
        }
   ###2.条件查询接口
      Future<BaasDBGetObjResult<T>> getObjects<T extends BaasObject>(Map<String, dynamic> queryCondition, int skip, int limit, String orderBy, Function createT,{int count: 0,Duration timeout : baasDBTimeout});
   ####查询条件：
        将需要过滤的条件和其取值/取值范围 以 Key:value的形式写入Map，
        比如：
        1. 查询name是"李四"的记录
            {"name": "李四"}
        2. 查询playerName是Tim0，dateTime大于等于1000，小于3000的记录
            {"playerName": "Tim0","dateTime":{"\$gte": 1000, "\$lte": 3000}}
        3. 查询分数等于1，3，5，7，9的记录
            {"score": {"in": [1, 3, 5, 7, 9]}}
        4. 查询名字不为 Jonathan Walsh，Dario Wunsch，Shawn Simon 的记录
            {"playerName": 
                {"nin": ["Jonathan Walsh", "Dario Wunsch", "Shawn Simon"]}
            }
        5. 查询含有符合子查询条件的记录
            {"hometown":{"$select":
              {"query":{"className":"Team","where":{"winPct":{"$gt":0.5}}},"key":"city"}}
             }
            }
        6. 组合查询
            {"or": [
                {"wins": {"$gt": 150}}, 
                {"wins": {"lt": 5}}
              ]
            }
        查询条件支持的操作有：
            关键词	            操作
            $lt	                少于
            $lte	            少于等于
            $gt	                多于
            $gte	            多于等于
            $ne	                不等于
            $in	                包含
            $nin	            不包含
            $exists	            存在某个字段
            $select	            符合子查询条件
            $dontSelect	        不符合子查询条件
            $all	            全部含有
            $regex	            某个字段的值符合相应的正则
            $text	            在有索引的字段上执行文字匹配
   #### skip(起始间隔数量),limit(限制每次返回数量)
        查询结果的偏移量和获取数量。
        一次查询过多数据，会影响性能，且业务不需要，则可以按照业务需求（分页显示）分片多次查询
   #### orderBy（查询结果排序）
        排序字段可以是一个或者多个字段的组合。
        比如 返回结果以 score 字段升序排列，name 降序排列的记录
        score,-name
   #### 数据结构和创建函数：
        T为继承自BaasObject的结构，且需要with mixin,比如：
        class TestDB extends BaasObject with TestDBMixin  {
          //  {"score": 0, "playerName": "Tim0", "cheatMode": false}
          int score;
          String playerName;
          bool cheatMode;
        }
        //Dart框架未引入反射，所以需要用户指定业务结构的创建方法
        function：createT:
            (){return new TestDB();}
   ####使用示例：
        BaasDBGetObjResult<TestDB> getResult = 
            await db.getObjects<TestDB>({"cheatMode": false}, 0, 6, "playerName", (){return new TestDB();},count:1);
         if(getResult != null && getResult.success){
         for( TestDB m in getResult.items){
                //业务处理逻辑
         }
         
   ###3. 数据创建接口   
      Future<BaasDBAddResult> addObject(BaasObject baasObject, {Duration timeout : baasDBTimeout});
   ####示例：
      BaasDBAddResult result = await db.addObject(new TestDB()
                ..score=1
                ..playerName="Tim1"
                ..cheatMode=false);
   ###4. 批量创建接口
      Future<List<BaasDBAddResult>> addObjects(List<BaasObject> items, {Duration timeout : baasDBTimeout});
   ####示例
            List<BaasDBAddResult> list = await db.addObjects([
              new TestDB()..score = 3..playerName="Tim3"..cheatMode=false,
              new TestDB()..score = 4..playerName="Tim4"..cheatMode=false,
              new TestDB()..score = 5..playerName="Tim5"..cheatMode=true
            ]);
   ###5. 更新接口
      Future<BaasDBUpdateResult> updateObjectById(BaasObject baasObject, {Duration timeout : baasDBTimeout}) ;
   ####示例：
        BaasDBUpdateResult updateResult = await db.updateObjectById(
            new TestDB()
            ..objectId = result.objectId
            ..score=333
        );
   ###6. count接口：
        count具有相同条件的记录数量
        Future<BaasDBCountResult> count(Map<String, dynamic> queryCondition,{Duration timeout : baasDBTimeout});
   ####示例：
        BaasDBCountResult result = await db.count({"cheatMode": false});
      //删除操作
   ###7. 删除接口
        删除功能提供了三种类型的接口
      1. 按数据ID删除，数据ID来源于get操作获得的数据
      Future< BaasDBDelResult > deleteById(String id,  {Duration timeout : baasDBTimeout});
      2. 按ID批量删除接口，items是数据ID的集合
        Future<List<BaasDBDelResult>> deleteManybyIds(List<String> items, {Duration timeout : baasDBTimeout}) ;
      3.按条件删除接口
      Future<BaasDBDelResult> delete(Map<String, dynamic> queryCondition, {Duration timeout : baasDBTimeout});
#半结构化数据接口
    半结构化数据是 结构化数据的一种形式，虽不符合关系型数据库或其他数据表的形式关联起来的数据模型结构，但包含相关标记，用来分隔语义元素以及对记录和字段进行分层，大多以JSON,XML来表示
    操作接口如下：
   ###1. DB初始化接口：参数是DB name
        DataBase getDB(String dbName);
   ####示例：
        获取DB接口操作对象，然后通过该DB对象操作数据接口
        if (mBaasSDK != null) {
          baasDB = mBaasSDK.getDB('GameScore');
        }
   ###2. 查询接口：
        查询接口跟结构化数据查询接口类似，queryCondition，skip, limit, orderBy跟结构化数据查询接口表示同意的含义。
        不同的是半结构化数据的查询返回结果基本单位是Json/Map.
        Future<BaasDBGetResult> get(Map<String, dynamic> queryCondition, int skip, int limit, String orderBy, {int count: 0,Duration timeout : baasDBTimeout});
   ####示例：
        BaasDBGetResult result = await db.get({"objectId": "${result.objectId}"}, 0, 6, "playerName");
   ###3. 创建数据接口
        paramJson：要创建的数据，用Map表示
      Future<BaasDBAddResult> add(Map<String,dynamic> paramJson, {Duration timeout : baasDBTimeout});
   ####示例：
        BaasDBAddResult result = await db.add({
                  "score":0,
                  "playerName":"tim0" ,
                  "cheatMode":false}
                );
   ###4. 批量创建数据接口
        items:要批量创建的数据，List<map>
      Future<List<BaasDBAddResult>> addMany(List items, {Duration timeout : baasDBTimeout});
   ####示例：
        List<BaasDBAddResult> results = await db.add(
                    [
                        { "score":0, "playerName":"tim0" , "cheatMode":false},
                        { "score":0, "playerName":"tim0" , "cheatMode":false}
                    ]
                 );
   ###5. 更新接口
      Future<BaasDBUpdateResult> updateById(String id, Map<String,dynamic> paramJson, {Duration timeout : baasDBTimeout}) ;
   ####示例：
       BaasDBUpdateResult result =  await db.updateById(result.objectId, {"cheatMode": true});
   ###6. upsert接口
        同其他存储引擎的upsert操作，如果不存在，就创建
      Future<BaasDBUpsertResult> upsert(Map<String,dynamic> queryParamJson, Map<String,dynamic> upsertParamJson, {Duration timeout : baasDBTimeout}) ;
   ####示例：
        Map<String, dynamic> query = <String, dynamic>{};
          query['userId'] = item['userId'];
          query['itemId'] = item['itemId'];
          query['provider'] = item['provider'];
          query['sort'] = item['sort'];
          query['date'] = item['date'];
       
          Map<String, dynamic> upsert = <String, dynamic>{};
          upsert['time'] = item['time'];
        
          BaasDBUpsertResult getResult = await db.upsert(query, upsert);
   ###7. 删除接口
        同结构化数据删除功能接口
#KV存储接口      
    KV存储接口主要存储非结构化数据（数据结构不规则或不完整，没有预定义的数据模型，不方便用数据库二维逻辑表来表现的数据），最常见的是应用Cache数据
    Baas模块提供了如下操作接口：
   ###1. init接口，参数是kv db名
        if (mBaasSDK != null) {
          db = mBaasSDK.getKVStore('XianYuItem');
        }
   ###2. put操作
        Future<KVStoreOpResult> put_async(String key,String value, {int kvStoreTimeout: 3600*1000,Duration apiTimeout : baasKVOpTimeout});
    参数含义：
        key: 数据存储的索引
        value:数据值
        kvStoreTimeout: 数据存储在db中的过期时间，默认一小时
        apiTimeout：操作超时时间，默认4秒
   ###3. remove操作 
        Future<KVStoreOpResult> remove_async(String key, {Duration apiTimeout : baasKVOpTimeout});
    参数含义：
        key:数据存储在DB中的唯一索引
        kvStoreTimeout: 数据存储在db中的过期时间，默认一小时
        apiTimeout：操作超时时间，默认4秒
   ###4. get操作 查询DB中key对应的值       
        Future<KVStoreOpResult> get_async(String key, {Duration apiTimeout : baasKVOpTimeout});
    参数含义：
        key: 数据存储的索引
        kvStoreTimeout: 数据存储在db中的过期时间，默认一小时
        apiTimeout：操作超时时间，默认4秒
   ###5. 批量操作接口
    除了单个数据的操作接口，还提供了批量删除接口，可以批量
     Future<Map<String,KVStoreOpResult>> mput_async(Map<String,String> map,{int kvStoreTimeout: 3600*1000,int version : 0, Duration apiTimeout:baasKVOpTimeout});
     Future<Map<String,KVStoreOpResult>> mget_async(List<String> keys,{ Duration apiTimeout:baasKVOpTimeout});
     Future<Map<String,KVStoreOpResult>> mremove_async(List<String> keys,{ Duration apiTimeout:baasKVOpTimeout});
#扩展Baas功能
    Baas模块默认实现了阿里云Baas服务接口，但是提供给了扩展能力，开发同学可以根据自己的需要接入不同实现的存储服务。
    只要这些服务的接口实现了database.dart和kv_store.dart中提供的接口能力即可。