
import 'dart:convert';

import 'package:crclib/crclib.dart';
import 'package:saker/framework/baas/aliyun/config/baas_config.dart';

import '../../interfaces/kv_store.dart';
import 'package:saker/framework/environment/env_base.dart';
import 'package:saker/framework/log/log.dart';
import 'package:saker/framework/tair/dart_tair.dart';

class AliYunKVStore implements KVStore {

  String _kvName;
  Logger logger;
  String _prefix;
  static TairService baasKVInstance;

  AliYunKVStore(this._kvName, this.logger){
    if(baasKVInstance == null){
      //根据key 查询switch配置中的tair Name.
      baasKVInstance = new TairService(!EnvFramework.isTesting() ? "36d381a4e5594307": 'd', 1000, 30000,150);
      _prefix =   new Crc32Zlib().convert(utf8.encode(_kvName)).toString();
    }
  }

  String get name => _kvName;

  String addKeyPrefix(String key){
    return baasSDKConfigAliyun.appID+_kvName+"_"+key;
  }

  String trimKeyPrefix(String key){
    return key.substring(baasSDKConfigAliyun.appID.length+_kvName.length+1);//_kvName长度+"_"长度
  }
  
  Future<KVStoreOpResult> put_async(String key,String value, {int kvStoreTimeout: 3600*1000, Duration apiTimeout : baasKVOpTimeout})async{
    TairOpResult tairOpResult = await baasKVInstance.put_async(addKeyPrefix(key), value, tairCacheTimeout: kvStoreTimeout,apiTimeout: apiTimeout);
    return new KVStoreOpResult()..resCode = tairOpResult.resCode..result = tairOpResult.result;
  }
  Future<KVStoreOpResult> remove_async(String key, {Duration apiTimeout : baasKVOpTimeout})async{
    TairOpResult tairOpResult = await baasKVInstance.remove_async(addKeyPrefix(key),apiTimeout: apiTimeout);
    return new KVStoreOpResult()..resCode = tairOpResult.resCode..result = tairOpResult.result;
  }
  Future<KVStoreOpResult> get_async(String key, {Duration apiTimeout : baasKVOpTimeout})async{
    TairOpResult tairOpResult = await baasKVInstance.get_async(addKeyPrefix(key),apiTimeout: apiTimeout);
    return new KVStoreOpResult()..resCode = tairOpResult.resCode..result = tairOpResult.result;
  }

  Future<Map<String,KVStoreOpResult>> mget_async(List<String> keys,{ Duration apiTimeout:baasKVOpTimeout})async {
    Map<String, KVStoreOpResult> baasResult = {};
    if(keys.length <= 0){
      return baasResult;
    }
    List<String> baasKeys = [];
    keys.forEach((String key){
      baasKeys.add(addKeyPrefix(key));
    });
    Map<String,TairOpResult> results = await baasKVInstance.mget_async(baasKeys,apiTimeout: apiTimeout);
    results.forEach((String key,TairOpResult value){
      baasResult[trimKeyPrefix(key)] = new KVStoreOpResult()..result = value.result..resCode = value.resCode;
    });
    return baasResult;
  }
  Future<Map<String,KVStoreOpResult>> mremove_async(List<String> keys,{ Duration apiTimeout:baasKVOpTimeout})async{
    Map<String, KVStoreOpResult> baasResult = {};
    if(keys.length <= 0){
      return baasResult;
    }
    List<String> baasKeys = [];
    keys.forEach((String key){
      baasKeys.add(addKeyPrefix(key));
    });
    Map<String,TairOpResult> results = await baasKVInstance.mremove_async(baasKeys,apiTimeout: apiTimeout);
    results.forEach((String key,TairOpResult value){
      baasResult[trimKeyPrefix(key)] = new KVStoreOpResult()..result = value.result..resCode = value.resCode;
    });
    return baasResult;
  }
  Future<Map<String,KVStoreOpResult>> mput_async(Map<String,String> map,{int kvStoreTimeout: 3600*1000,int version : 0, Duration apiTimeout:baasKVOpTimeout})async{
    Map<String, KVStoreOpResult> baasResult = {};
    if(map.length <= 0){
      return baasResult;
    }
    Map<String,String> baasMap={};
    map.forEach((String key,String value){
      baasMap[addKeyPrefix(key)] = value;
    });
    Map<String,TairOpResult> results = await baasKVInstance.mput_async(baasMap,apiTimeout: apiTimeout,tairCacheTimeout: kvStoreTimeout);
    results.forEach((String key,TairOpResult value){
      baasResult[trimKeyPrefix(key)] = new KVStoreOpResult()..result = value.result..resCode = value.resCode;
    });
    return baasResult;
  }
}