
import 'dart:convert';

import './baas_object.dart';

const Duration baasDBTimeout = const Duration(seconds: 4);

/*
* 统一各Baas产品的数据库接口
* */
class BaasDBGetObjResult<T> {
  bool success;
  int totalCount;
  List< T > items;//业务无关结构，业务需要自己调用fromJson解析items中的结构
  String errMsg;
  String toString(){
    return "BaasDBGetResult: succ:$success, errMsg:$errMsg, totalCnt:$totalCount items:${json.encode(items)}";
  }
}

class BaasDBGetResult {
  bool success;
  int totalCount;
  List< Map<String, dynamic> > items;//业务无关结构，业务需要自己调用fromJson解析items中的结构
  String errMsg;
  String toString(){
    return "BaasDBGetResult: succ:$success, errMsg:$errMsg, totalCnt:$totalCount items:${json.encode(items)}";
  }
}

class BaasDBAddResult{
  bool success;
  String errMsg;
  String objectId;
  String createAt;

  String toString(){
    return "BaasDBAddResult: succ:$success, errMsg:$errMsg, objectId:$objectId, createAt:$createAt ";
  }
}

class BaasDBUpdateResult{
  bool success;
  String errMsg;
  String objectId;
  String updateAt;

  String toString(){
    return "BaasDBUpdateResult: succ:$success, errMsg:$errMsg, objectId:$objectId, createAt:$updateAt ";
  }
}

class BaasDBUpsertResult{
  bool success;
  String errMsg;
  String updateAt;

  String toString(){
    return "BaasDBUpsertResult: succ:$success, errMsg:$errMsg, createAt:$updateAt ";
  }
}


class BaasDBDelResult{
  bool success;
  String errMsg;
  String toString(){
    return "BaasDBDelResult: succ:$success, errMsg:$errMsg ";
  }
}

class BaasDBCountResult{
  bool success;
  String errMsg;
  int totalCount;
  String toString(){
    return "BaasDBDelResult: succ:$success, errMsg:$errMsg totalCnt:$totalCount";
  }
}

abstract class DataBase{

  String get name;

  //如果count设定为1，则在返回结果中的totalCount会显示符合该条件的结果总数，默认是0
  Future<BaasDBGetResult> get(Map<String, dynamic> queryCondition, int skip, int limit, String orderBy, {int count: 0,Duration timeout : baasDBTimeout});
  Future<BaasDBAddResult> add(Map<String,dynamic> paramJson, {Duration timeout : baasDBTimeout});
  Future<List<BaasDBAddResult>> addMany(List items, {Duration timeout : baasDBTimeout});
  Future<BaasDBUpdateResult> updateById(String id, Map<String,dynamic> paramJson, {Duration timeout : baasDBTimeout}) ;
  Future<BaasDBUpsertResult> upsert(Map<String,dynamic> queryParamJson, Map<String,dynamic> upsertParamJson, {Duration timeout : baasDBTimeout}) ;

  //结构化数据操作接口
  Future<BaasDBGetObjResult<T>> getObjects<T extends BaasObject>(Map<String, dynamic> queryCondition, int skip, int limit, String orderBy, Function createT,{int count: 0,Duration timeout : baasDBTimeout});
  Future<BaasDBAddResult> addObject(BaasObject baasObject, {Duration timeout : baasDBTimeout});
  Future<List<BaasDBAddResult>> addObjects(List<BaasObject> items, {Duration timeout : baasDBTimeout});
  Future<BaasDBUpdateResult> updateObjectById(BaasObject baasObject, {Duration timeout : baasDBTimeout}) ;

  //count操作
  Future<BaasDBCountResult> count(Map<String, dynamic> queryCondition,{Duration timeout : baasDBTimeout});

  //删除操作
  Future< BaasDBDelResult > deleteById(String id,  {Duration timeout : baasDBTimeout});
  Future<List<BaasDBDelResult>> deleteManybyIds(List<String> items, {Duration timeout : baasDBTimeout}) ;
  Future<BaasDBDelResult> delete(Map<String, dynamic> queryCondition, {Duration timeout : baasDBTimeout});
}