import 'dart:convert';
import 'package:saker/common/utils/type_cast.dart';
import 'package:saker/framework/log/log.dart';
import 'package:saker/framework/switch/dart_switch_extension.dart';
import 'package:saker/framework/switch/dart_switch_utils.dart';
import 'package:dcache/dcache.dart';

class FrameWorkSwitchManager{
  static Map<String ,SwitchDataType> switch_var_type = {};
  static Cache cache = new LruCache(storage: new SimpleStorage(size: 400))
    ..expiration = new Duration(seconds: 3)
    ..syncLoading = true
    ..loader = (k, value){
      String key = safeCast(k);
      if(!switch_var_type.containsKey(key)) {
        MyLogger.getLogger().error("SwitchManager not fond key start:${key}");
        return null;
      }
      MyLogger.getLogger().debug("SwitchManager get from c++: key=${key}, type=${switch_var_type[key]}");
      SwitchDataType key_type = switch_var_type[key];
      switch(key_type) {
      case SwitchDataType.type_string:
        List list = switch_get_val_string(key);
        if (list.length == 2 && list[0] == 0) {
          return list[1];
        } else {
          MyLogger.getLogger().error("SwitchManager get val failed:${key} ${list}");
        }
        break;
      case SwitchDataType.type_int:
        List list = switch_get_val_int(key);
        if (list.length == 2 && list[0] == 0) {
          return list[1];
        } else {
          MyLogger.getLogger().error("SwitchManager get val failed:${key} ${list}");
        }
        break;
      case SwitchDataType.type_bool:
        List list = switch_get_val_bool(key);
        if (list.length == 2 && list[0] == 0) {
          return list[1];
        } else {
          MyLogger.getLogger().error("SwitchManager get val failed:${key} ${list}");
        }
        break;
      case SwitchDataType.type_map:
        List list = switch_get_val_string(key);
        if (list.length == 2 && list[0] == 0) {
          try {
            var m = json.decode(list[1] as String);
            if (m is Map) {
              return m;
            }
          } catch (e) {
            MyLogger.getLogger().error("SwitchManager get val faild: key=${key}, error=${e}}");
          }
        }
        MyLogger.getLogger().error("SwitchManager get val failed:${key} ${list}");
        break;
      case SwitchDataType.type_list:
        List list = switch_get_val_string(key);
        if (list.length == 2 && list[0] == 0) {
          try {
            var m = json.decode(list[1] as String);
            if (m is List) {
              return m;
            }
          } catch (e) {
            MyLogger.getLogger().error("SwitchManager get val faild: key=${key}, error=${e}}");
          }
        }
        MyLogger.getLogger().error("SwitchManager get val failed:${key} ${list}");
        break;
      case SwitchDataType.type_set:
        List list = switch_get_val_string(key);
        if (list.length == 2 && list[0] == 0) {
          try {
            var m = json.decode(list[1] as String);
            if (m is List) {
              return m.toSet();
            }
          } catch (e) {
            MyLogger.getLogger().error("SwitchManager get val faild: key=${key}, error=${e}}");
          }
        }
        MyLogger.getLogger().error("SwitchManager get val failed:${key} ${list}");
        break;
      default:
        MyLogger.getLogger().error("SwitchManager unsupported type. ${key_type}");
      }
    }
  ;
	static void Register() {
		try {
			switch_bind_string("betaHostIp", '''Beta机器IP''', 1, '''''');
			switch_var_type["betaHostIp"] = SwitchDataType.type_string;

			switch_bind_string("betaUserNicks", '''重路由用户名单''', 1, '''[]''');
			switch_var_type["betaUserNicks"] = SwitchDataType.type_list;

      switch_bind_int("serviceMeshRate", '''ServiceMesh切流比例,默认0，不切流''', 2, 0);
      switch_var_type["serviceMeshRate"] = SwitchDataType.type_int;

      switch_bind_string("baasHost", '''线上Baas服务地址''', 1, "baas-service-aliyun-com.vipserver");
      switch_var_type["baasHost"] = SwitchDataType.type_string;

      switch_bind_string("preBaasHost", '''预发环境Baas服务地址''', 1, "pre-baas-service-aliyun-com.vipserver");
      switch_var_type["preBaasHost"] = SwitchDataType.type_string;
		} catch (e) {
			MyLogger.getLogger().error("SwitchManager Register faild. ...${e.toString()}....}");
		}
	}

	static String get betaHostIp => safeCast<String>(cache.get("betaHostIp"));

	static String get baasHost => safeCast<String>(cache.get("baasHost"));

	static String get preBaasHost => safeCast<String>(cache.get("preBaasHost"));

	static List get betaUserNicks => safeCast<List>(cache.get("betaUserNicks"))??[];

  static int get serviceMeshRate => safeCast<int>(cache.get("serviceMeshRate"))??0;

}