//初始化Isolate环境
import 'dart:io';

import 'package:df_api/df_func.dart';
import 'package:saker/framework/environment/env_type.dart';
import 'package:saker/framework/services/service_base.dart';
import 'package:saker/framework/configfile/configfile.dart';
import 'package:saker/framework/services/service_server.dart';
import 'package:saker/framework/services/service_var.dart';




typedef EnvBase EnvFunc();

class EnvFramework {
  static String rootPath = '';
  static String appName = '';
  static int env = 1;
  static String envStr = 'daily';
  static String monitorLogFile = 'dartMonitor.log';

  static bool isTesting() {
    return env <= ENV_DAILY;
  }

  static String ossConf = 'oss.conf';
  static String ossBucket = 'oss://fish-dart';

  static int hsfWorkerThreadCnt = 8;
  static int hsfIOThreadCnt = 4;
  static int hsfLogLevel = 5;
  static String hsfLogPath = '';
  static Duration hsfConsumeTimeout = new Duration(milliseconds: 4000);
  static String hsfSnapshotPath = '/tmp/snapshot';

  //Tair
  static String tairLogPath = '';
  static String tairLogLevel = 'WARN';
  static int tairIOThreadCnt = 2;

  //Observe
  static String observeAddr = '127.0.0.1';

  static int observePort = 8000;

  @Deprecated('to be replaced by serverAddress')
  static String httpAddr = '0.0.0.0';
  static String serverAddress = '0.0.0.0';

  @Deprecated('to be replaced by serverPort')
  static int httpPort = 8080;
  static int serverPort = 8080;
  @Deprecated('to be replaced by concurrency')
  static int httpConcurrence = 2;

  static int concurrence = 2;

  static String serviceServerKind = 'http';

  static int hotReloadInterval = 0;

  @Deprecated('to be replaced by isolateStartDelay')
  static Duration httpDelay = new Duration(milliseconds: 100);

  static Duration isolateStartDelay = new Duration(milliseconds: 100);

  static Duration hsfProviderDelay = new Duration(milliseconds: 2000);

  //日常环境log收集服务VIP,对应到 11.162.156.152（dartserverpretest）端口80 http://100.67.0.200/dartserverLog 在预发上配置项
  static String logServerUrl = '';

  static bool isHotReload = false;
  static bool isFaaS = false;
  static bool isPluginActived = false;

  static String version = '1.0';

  String _a;

  static void init(String root) {
    rootPath = root;

    File localFile = new File('${root}/framework.conf');
    if (localFile.existsSync()) {
      List<String> lines = localFile.readAsLinesSync();
      Config localConf = new Config.fromStrings(lines);
      Map<String, String> defaults = localConf.defaults();

      if (defaults.containsKey('appName')) {
        appName = defaults['appName'];
      }

      if (defaults.containsKey('envStr')) {
        envStr = defaults['envStr'];
      }

      if (defaults.containsKey('env')) {
        env = int.tryParse(defaults['env']);
      }

      if (defaults.containsKey('monitorLogFile')) {
        monitorLogFile = defaults['monitorLogFile'];
      }

      if (defaults.containsKey('ossConf')) {
        ossConf = defaults['ossConf'];
      }

      if (defaults.containsKey('ossBucket')) {
        ossBucket = defaults['ossBucket'];
      }

      if (defaults.containsKey('hotReloadInterval')) {
        hotReloadInterval = int.tryParse(defaults['hotReloadInterval'])??0;
      }

      if (defaults.containsKey('hsfWorkerThreadCnt')) {
        hsfWorkerThreadCnt = int.tryParse(defaults['hsfWorkerThreadCnt']);
      }

      if (defaults.containsKey('hsfIOThreadCnt')) {
        hsfIOThreadCnt = int.tryParse(defaults['hsfIOThreadCnt']);
      }

      if (defaults.containsKey('hsfLogLevel')) {
        hsfLogLevel = int.tryParse(defaults['hsfLogLevel']);
      }

      if (defaults.containsKey('hsfLogPath')) {
        hsfLogPath = defaults['hsfLogPath'];
      }

      if (defaults.containsKey('hsfConsumeTimeout')) {
        hsfConsumeTimeout = new Duration(milliseconds: int.tryParse(defaults['hsfConsumeTimeout'])??4000);
      }

      if (defaults.containsKey('hsfSnapshotPath')) {
        hsfSnapshotPath = defaults['hsfSnapshotPath'];
      }

      if (defaults.containsKey('tairLogPath')) {
        tairLogPath = defaults['tairLogPath'];
      }

      if (defaults.containsKey('tairLogLevel')) {
        tairLogLevel = defaults['tairLogLevel'];
      }

      if (defaults.containsKey('tairIOThreadCnt')) {
        tairIOThreadCnt = int.tryParse(defaults['tairIOThreadCnt'])??2;
      }

      if (defaults.containsKey('observeAddr')) {
        observeAddr = defaults['observeAddr'];
      }

      if (defaults.containsKey('observePort')) {
        observePort = int.tryParse(defaults['observePort'])??8000;
      }

      //muti-server-kind compatible
      if (defaults.containsKey('httpAddr')) {
        httpAddr = defaults['httpAddr'];
        serverAddress = httpAddr;
      }

      if (defaults.containsKey('httpPort')) {
        httpPort = int.tryParse(defaults['httpPort']);
        serverPort = httpPort;
      }

      if (defaults.containsKey('serverAddress')) {
        serverAddress = defaults['serverAddress'];
        httpAddr = serverAddress;
      }

      if (defaults.containsKey('serverPort')) {
        serverPort = int.tryParse(defaults['serverPort']);
        httpPort = serverPort;
      }

      if (defaults.containsKey('httpConcurrence')) {
        httpConcurrence = int.tryParse(defaults['httpConcurrence'])??1;
        if (httpConcurrence <= 0) {
          httpConcurrence = 1;
        }
        concurrence = httpConcurrence;
      }

      if (defaults.containsKey('concurrence')) {
        concurrence = int.tryParse(defaults['concurrence'])??1;
        if (concurrence <= 0) {
          concurrence = 1;
        }
        httpConcurrence = concurrence;
      }

      if (defaults.containsKey('httpDelay')) {
        httpDelay = new Duration(milliseconds: int.tryParse(defaults['httpDelay']));
        isolateStartDelay = httpDelay;
      }

      if (defaults.containsKey('isolateStartDelay')) {
        isolateStartDelay = new Duration(milliseconds: int.tryParse(defaults['httpDelay']));
        httpDelay = isolateStartDelay;
      }

      if (defaults.containsKey('hsfProviderDelay')) {
        hsfProviderDelay = new Duration(milliseconds: int.tryParse(defaults['hsfProviderDelay']));
      }

      if (defaults.containsKey('logServerUrl')) {
        logServerUrl = defaults['logServerUrl'];
      }

      if(defaults.containsKey("isPluginActived")){
        isPluginActived = defaults['isPluginActived'].toLowerCase() == "true" ? true : false;
      }
    }
  }
}

abstract class EnvBase {
  static EnvFunc getEnv = null;

  @deprecated
  Map<String, Processing> getPreProcessing() => {};

  @deprecated
  Map<String, ServiceBase> getServiceEntries();

//  FunctionService getFunctionService();

  void init(String root, String tag);

  //环境初始化
  void initApp();

  //新建一个Isolate要做的事情
  void initIsolate();

  //退出Isolate 释放的资源
  void releaseIsolate();

  //退出进程要做的
  void closeApp();
}


