
import 'dart:async';
import 'dart:isolate';

import 'package:saker/framework/log/log.dart';

class EventItem {
  int id;
  SendPort sendPort;
  Future<dynamic> future;
}

final int EventType_Any = 0;
final int EventType_Notice = 1; //通知，封装数据包
final int EventType_Hsf_Pull = 5; //HSF数据拉取
final int EventType_Hsf_Req = 8; //HSF数据拉取
final int EventType_Tair_Pull = 6; //Tair 数据拉取
final int EventType_Traffic_Guard = 7; //Traffic guard 通知
final int EventStatus_OK = 0;

typedef void event_callback(dynamic messages);

class EventPoolBase {
  ReceivePort receivePort = null;
  bool health = true;
  int _taskId = 1;
  Map<int,Completer> completerMap = {};

  void doTaskReal(var msg) {
    int taskId = msg[0] as int;
    if (!completerMap.containsKey(taskId)) {
      MyLogger.getLogger().error("EventPoolBase EventId not found: EventID=$taskId, succ:${msg[1]} Msg:${msg[2]} ");
      return;
    }
    Completer completer = completerMap.remove(taskId);
    if (msg[1] == EventStatus_OK) {
      completer.complete(msg[2]);
    } else {
      completer.completeError("${msg[1]}:  ${msg[2]}");
      MyLogger.getDebugLogger()?.debug("EventPoolBase Completer complete failed: EventID=$taskId, succ:${msg[1]} Msg:${msg[2]} ");
    }
  }

  EventItem getEventReal() {
    var id = _taskId++;
    Completer _completer = new Completer();
    completerMap[id] = _completer;

    return new EventItem()
      ..id = id
      ..sendPort = receivePort.sendPort
      ..future = _completer.future;
  }
}




