import 'dart:typed_data';

import 'package:saker/framework/hessian/hessian2_const.dart';
import 'package:saker/framework/hessian/hessian_object_base.dart';

class ChunkBlock {
  int bytesLen = 0;
  Uint8List uint8list;
}

class ChunkBlockPool{
  List<ChunkBlock> idleChunk;

  ChunkBlockPool(){
    idleChunk = [];

  }
  void delChunk(ChunkBlock chunkBlock){
    idleChunk.add(chunkBlock);
  }

  ChunkBlock getChunk(){
    if(idleChunk.isEmpty){
      ChunkBlock chunkBlock = new ChunkBlock()
        ..uint8list = new Uint8List(SIZE)
        ..bytesLen = 0;
      idleChunk.add(chunkBlock);
    }
    return idleChunk.removeLast()..bytesLen = 0;
  }
}

int SIZE = 8 * 1024;

class Hessian1Output {
  // map of references
  Map<Object,int> _refs ;
  ByteData _byteData;
  ChunkBlock _chunkBlock;

  List<ChunkBlock> _chunkBlocks = new List();
  int _totalBytes = 0;
  static int NULL = 0xdeadbeef;

  Hessian1Output(){
    _chunkBlock = new ChunkBlock()
      ..uint8list = new Uint8List(SIZE)
      ..bytesLen = 0;
    _byteData = new ByteData.view(_chunkBlock.uint8list.buffer);
    _chunkBlock.bytesLen = 0;
    _chunkBlocks.add(_chunkBlock);
    _totalBytes = 0;
    _refs = {};
  }

  void _flushBuffer(){
    _chunkBlock = new ChunkBlock()
      ..uint8list = new Uint8List(SIZE)
      ..bytesLen = 0;
    _byteData = new ByteData.view(_chunkBlock.uint8list.buffer);
    _chunkBlock.bytesLen = 0;
    _chunkBlocks.add(_chunkBlock);
  }

  void writeNull(){
    if (SIZE < _chunkBlock.bytesLen +1) {
      _flushBuffer();
    }
    _byteData.setUint8(_chunkBlock.bytesLen++, BC_NULL);
  }

  void writeBoolean(bool value){
    if (SIZE < _chunkBlock.bytesLen + 1) {
      _flushBuffer();
    }

    if (value)
      _byteData.setUint8(_chunkBlock.bytesLen++, BC_TRUE); //T
    else
      _byteData.setUint8(_chunkBlock.bytesLen++, BC_FALSE); //F
  }

  void writeInt32(int val){
    if(SIZE < _chunkBlock.bytesLen + 5){
      _flushBuffer();
    }
    _byteData.setUint8(_chunkBlock.bytesLen++, BC_INT);
    _byteData.setUint32(_chunkBlock.bytesLen, val);
    _chunkBlock.bytesLen += 4;
  }

  void writeLong(int val){
    if(SIZE < _chunkBlock.bytesLen + 9){
      _flushBuffer();
    }
    _byteData.setUint8(_chunkBlock.bytesLen++, BC_LONG);
    _byteData.setUint64(_chunkBlock.bytesLen, val);
    _chunkBlock.bytesLen += 8;
  }

  void writeDouble(double val){
    if(SIZE < _chunkBlock.bytesLen + 9){
      _flushBuffer();
    }
    _byteData.setUint8(_chunkBlock.bytesLen++, BC_DOUBLE);
    _byteData.setFloat64(_chunkBlock.bytesLen, val);
    _chunkBlock.bytesLen += 8;
  }

  void writeUTCDate(DateTime dateTime){
    if(SIZE < _chunkBlock.bytesLen + 9){
      _flushBuffer();
    }

    int time = dateTime.millisecondsSinceEpoch;
    _byteData.setUint8(_chunkBlock.bytesLen++, 100); // 100 : 'd'
    _byteData.setUint64(_chunkBlock.bytesLen, time);
    _chunkBlock.bytesLen += 8;
  }

  /**
   * Prints a string to the stream, encoded as UTF-8
   *
   * @param v the string to print.
   */
  void printString3(String v, int strOffset, int length) {

    for (int i = 0; i < length; i++) {
      if (SIZE <= _chunkBlock.bytesLen + 16) {
        _flushBuffer();
      }

      //char ch = v.charAt(i + strOffset);
      int ch = v.codeUnitAt(i + strOffset);

      if (ch < 0x80) {
        _byteData.setUint8(_chunkBlock.bytesLen++, ch);
      }
      else if (ch < 0x800) {
        _byteData.setUint8(_chunkBlock.bytesLen++, (0xc0 + ((ch >> 6) & 0x1f)));
        _byteData.setUint8(_chunkBlock.bytesLen++, (0x80 + (ch & 0x3f)));
      }
      else {
        _byteData.setUint8(_chunkBlock.bytesLen++, (0xe0 + ((ch >> 12) & 0xf)));
        _byteData.setUint8(_chunkBlock.bytesLen++, (0x80 + ((ch >> 6) & 0x3f)));
        _byteData.setUint8(_chunkBlock.bytesLen++, (0x80 + (ch & 0x3f)));
      }
    }
  }

  void writeType(String type){
    if (type == null) {
      writeNull();
    }else{
      if (SIZE <= _chunkBlock.bytesLen + 16) {
        _flushBuffer();
      }

      int length = type.length;
      _byteData.setUint8(_chunkBlock.bytesLen++, 116);//'t'
      _byteData.setUint8(_chunkBlock.bytesLen++, (length >> 8));
      _byteData.setUint8(_chunkBlock.bytesLen++, length);
      printString3(type, 0, length);
    }
  }

  void writeString(String value){
    if (value == null) {
      writeNull();
    }
    else {
      int length = value.length;
      int strOffset = 0;

      while (length > 0x8000) {
        int sublen = 0x8000;

        if (SIZE <= _chunkBlock.bytesLen + 16) {
          _flushBuffer();
        }

        // chunk can't end in high surrogate
//        char tail = value.charAt(offset + sublen - 1);
        int tail = value.codeUnitAt(strOffset + sublen - 1);

        if (0xd800 <= tail && tail <= 0xdbff)
          sublen--;

        _byteData.setUint8(_chunkBlock.bytesLen++, H1_BC_STRING_CHUCK);//'s'
        _byteData.setUint8(_chunkBlock.bytesLen++, (sublen >> 8));
        _byteData.setUint8(_chunkBlock.bytesLen++, sublen);
        printString3(value, strOffset, sublen);

        length -= sublen;
        strOffset += sublen;
      }

      if (SIZE <= _chunkBlock.bytesLen + 16) {
        _flushBuffer();
      }

      _byteData.setUint8(_chunkBlock.bytesLen++, BC_STRING);//'S'
      _byteData.setUint8(_chunkBlock.bytesLen++, (length >> 8));
      _byteData.setUint8(_chunkBlock.bytesLen++, length);
      printString3(value, strOffset, length);
    }
  }

  //TODO:char array???????


  /**
   * Writes a byte array to the stream.
   * The array will be written with the following syntax:
   *
   * <code><pre>
   * B b16 b18 bytes
   * </pre></code>
   *
   * If the value is null, it will be written as
   *
   * <code><pre>
   * N
   * </pre></code>
   *
   * @param value the string value to write.
   */
  void writeBytes(Uint8List buffer) {
    if (buffer == null) {
      writeNull();
    }
    else
      writeBytes3(buffer, 0, buffer.length);
  }


  /**
   * Writes a byte array to the stream.
   * The array will be written with the following syntax:
   *
   * <code><pre>
   * B b16 b18 bytes
   * </pre></code>
   *
   * If the value is null, it will be written as
   *
   * <code><pre>
   * N
   * </pre></code>
   *
   * @param value the string value to write.
   */
  void writeBytes3(Uint8List buffer, int offset, int length) {
    if (buffer == null) {
      writeNull();
    }
    else {
      while (length > 0x8000) {
        int sublen = 0x8000;

        if (SIZE < _chunkBlock.bytesLen +3) {
          _flushBuffer();
        }
        _byteData.setUint8(_chunkBlock.bytesLen++, 98);//'b'
        _byteData.setUint8(_chunkBlock.bytesLen++, sublen >> 8);
        _byteData.setUint8(_chunkBlock.bytesLen++, sublen);

        //将 0x8000个byte写入到多个_chunkBlock中。
        int byteLeft = sublen;
        while(byteLeft > 0){
          int chunkSpaceSizeLeft = SIZE - _chunkBlock.bytesLen;
          _chunkBlock.uint8list.setRange(_chunkBlock.bytesLen, _chunkBlock.bytesLen + chunkSpaceSizeLeft, buffer, offset);
          _chunkBlock.bytesLen += chunkSpaceSizeLeft;
          byteLeft -= chunkSpaceSizeLeft;

          _flushBuffer();
        }

        length -= sublen;
        offset += sublen;
      }

      if (SIZE < _chunkBlock.bytesLen +3) {
        _flushBuffer();
      }
      _byteData.setUint8(_chunkBlock.bytesLen++, 66 );//'B'
      _byteData.setUint8(_chunkBlock.bytesLen++, length >> 8);
      _byteData.setUint8(_chunkBlock.bytesLen++, length);
//      os.write(buffer, offset, length);
      int byteLeft = length;
      while(byteLeft > 0){
        int chunkSpaceSizeLeft = SIZE - _chunkBlock.bytesLen;
        _chunkBlock.uint8list.setRange(_chunkBlock.bytesLen, _chunkBlock.bytesLen + chunkSpaceSizeLeft, buffer, offset);
        _chunkBlock.bytesLen += chunkSpaceSizeLeft;
        byteLeft -= chunkSpaceSizeLeft;
        _flushBuffer();
      }
    }
  }

  /**
   * Writes a reference.
   *
   * <code><pre>
   * R b32 b24 b16 b8
   * </pre></code>
   *
   * @param value the integer value to write.
   */
  void writeRef(int value){
    if (SIZE < _chunkBlock.bytesLen + 5)
      _flushBuffer();

    _byteData.setUint8(_chunkBlock.bytesLen++, 82); //'R'

    _byteData.setUint32(_chunkBlock.bytesLen, value);
    _chunkBlock.bytesLen += 4;
  }

  //int32
  void writeInt( int value){
    writeInt32(value);
  }


  /**
   * If the object has already been written, just write its ref.
   *
   * @return true if we're writing a ref.
   */
  bool addRef(Object object){
    if (_refs == null)
      _refs = {};

    int ref = _refs[object];

    if (ref != null) {
      writeRef(ref);
      return true;
    }
    else {
      _refs[object] = _refs.length;
      return false;
    }
  }

  int getRef(Object obj) {
    int value;

    if (_refs == null)
      return -1;

    value = _refs[obj];

    if (value == null)
      return -1;
    else
      return value;
  }

  /**
   * Resets the references for streaming.
   */
  void resetReferences()
  {
    if (_refs != null)
      _refs.clear();
  }

  /**
   * Removes a reference.
   */
  bool removeRef(Object obj){
    if (_refs != null) {
      _refs.remove(obj);
      return true;
    }
    else
      return false;
  }

  /**
   * Replaces a reference from one object to another.
   */
  bool replaceRef(Object oldRef, Object newRef){
    int value = _refs[oldRef];

    if (value != null) {
      _refs.remove(oldRef);
      _refs[newRef] = value;
      return true;
    }
    else
      return false;
  }


  /**
   * Prints a string to the stream, encoded as UTF-8 with preceeding length
   *
   * @param v the string to print.
   */
  void printLenString(String v){
    if(SIZE < _chunkBlock.bytesLen + 2){
      _flushBuffer();
    }
    if (v == null) {
      _byteData.setUint8(_chunkBlock.bytesLen++, 0	);
      _byteData.setUint8(_chunkBlock.bytesLen++, 0	);
    }
    else {
      int len = v.length;
      _byteData.setUint8(_chunkBlock.bytesLen++, len >> 8);
      _byteData.setUint8(_chunkBlock.bytesLen++, len);

      printString3(v, 0, len);
    }
  }

  /**
   * Writes the list header to the stream.  List writers will call
   * <code>writeListBegin</code> followed by the list contents and then
   * call <code>writeListEnd</code>.
   *
   * <code><pre>
   * V
   * t b16 b8 type
   * l b32 b24 b16 b8
   * </pre></code>
   */
  bool writeListBegin(int length, String type){
    if(SIZE < _chunkBlock.bytesLen + 2){
      _flushBuffer();
    }
    _byteData.setUint8(_chunkBlock.bytesLen++, 86	); //'V'

    String listType = type != null? type:"java.util.ArrayList";
    if(listType != "java.util.ArrayList" ) {
      _byteData.setUint8(_chunkBlock.bytesLen++, 116); //'t'

      printLenString(listType);
    }

    if (length >= 0) {
      if(SIZE < _chunkBlock.bytesLen + 5){
        _flushBuffer();
      }
      _byteData.setUint8(_chunkBlock.bytesLen++, 108	); //'l'
      _byteData.setUint8(_chunkBlock.bytesLen++, length >> 24);
      _byteData.setUint8(_chunkBlock.bytesLen++, length >> 16);
      _byteData.setUint8(_chunkBlock.bytesLen++, length >> 8);
      _byteData.setUint8(_chunkBlock.bytesLen++, length);
    }
    return true;
  }

  /**
   * Writes the tail of the list to the stream.
   */
  void writeListEnd(){
    if(SIZE < _chunkBlock.bytesLen + 1){
      _flushBuffer();
    }
    _byteData.setUint8(_chunkBlock.bytesLen++, 122);//'z'
  }

  /**
   * Writes the map header to the stream.  Map writers will call
   * <code>writeMapBegin</code> followed by the map contents and then
   * call <code>writeMapEnd</code>.
   *
   * <code><pre>
   * Mt b16 b8 (<key> <value>)z
   * </pre></code>
   */
  void writeMapBegin(String type){
    if(SIZE < _chunkBlock.bytesLen + 2){
      _flushBuffer();
    }
    _byteData.setUint8(_chunkBlock.bytesLen++, 77);//'M'

    String mapType = type != null? type:"java.util.HashMap";
    if(mapType != "java.util.HashMap"){
      _byteData.setUint8(_chunkBlock.bytesLen++, 116); //'t'
      printLenString(mapType);
    }
  }

  /**
   * Writes the tail of the map to the stream.
   */
  void writeMapEnd(){
    if(SIZE < _chunkBlock.bytesLen + 1){
      _flushBuffer();
    }
    _byteData.setUint8(_chunkBlock.bytesLen++, 122);//'z'
  }

  /**
   * Writes the object definition
   *
   * <code><pre>
   * C &lt;string> &lt;int> &lt;string>*
   * </pre></code>
   */
  int writeObjectBegin(String type) {
    if (SIZE < _chunkBlock.bytesLen + 32)
      _flushBuffer();

    _byteData.setUint8(_chunkBlock.bytesLen++, 77); //'M'

    writeType(type);

    return -1;
  }

  /**
   * Writes the tail of the map to the stream.
   */
  void writeObjectEnd(){
    if(SIZE < _chunkBlock.bytesLen + 1){
      _flushBuffer();
    }
    _byteData.setUint8(_chunkBlock.bytesLen++, 122);//'z'
  }

  /**
   * Writes any object to the output stream.
   */
  void writeObject(HessianObjectBase object,
      Map<String, generic_hessian_func> genericMap) {
    if (object == null) {
      return;
    }

    object.writeHessian1(this, genericMap);
  }


  void writeMapValue<K, V>(Map<K, V> val, void f(K k, V v)) {
    if (val == null) {
      writeNull();
      return;
    }

    if (addRef(val)) {
      return;
    }

    writeMapBegin(null);

    val.forEach((k, v) => f(k, v));

    writeMapEnd();
  }

  void writeListValue<E>(List<E> val, void f(E element)) {
    if (val == null) {
      writeNull();
      return;
    }

    if (addRef(val)) {
      return;
    }

    bool hasEnd = writeListBegin(val.length, null);

    val.forEach((i) => f(i));

    if (hasEnd) {
      writeListEnd();
    }
  }

  void writeSetValue<E>(Set<E> val, void f(E element)) {
    if (val == null) {
      writeNull();
      return;
    }

    if (addRef(val)) {
      return;
    }

    bool hasEnd = writeListBegin(val.length, 'java.util.HashSet');

    val.forEach((i) => f(i));

    if (hasEnd) {
      writeListEnd();
    }
  }

  void writeObjectValue(String type, dynamic val, Map<String, generic_hessian_func> genericMap) {
    if (type.startsWith('[')) {
      if (val == null) {
        writeNull();
        return;
      }

      if (addRef(val))
        return;

      if (type == '[byte') {
        writeBytes(val as Uint8List);
        return;
      }
      if (type == '[char') {
        writeString(val as String);
        return;
      }

      if (val is List) {
        bool hasEnd = writeListBegin(val.length, type);
        val.forEach((e) => writeObjectValue(type.substring(1), e, genericMap));

        if (hasEnd)
          writeListEnd();
      } else {
        writeNull();
      }
      return;
    }

    switch (type) {
      case 'java.lang.String':
      case 'String':
      case 'java.lang.Character':
        writeString(val as String);
        break;
      case 'boolean':
        if (val == null) {
          writeBoolean(false);
          break;
        }
        writeBoolean(val as bool);
        break;
      case 'java.lang.Boolean':
        if (val == null) {
          writeNull();
          break;
        }
        writeBoolean(val as bool);
        break;
      case 'byte':
      case 'short':
      case 'int':
        if (val == null) {
          writeInt(0);
          break;
        }
        writeInt(val as int);
        break;
      case 'java.lang.Byte':
      case 'java.lang.Short':
      case 'java.lang.Integer':
        if (val == null) {
          writeNull();
          break;
        }
        writeInt(val as int);
        break;
      case 'long':
        if (val == null) {
          writeLong(0);
          break;
        }
        writeLong(val as int);
        break;
      case 'java.lang.Long':
        if (val == null) {
          writeNull();
          break;
        }
        writeLong(val as int);
        break;
      case 'java.lang.Float':
      case 'java.lang.Double':
        if (val == null) {
          writeNull();
          break;
        }
        writeDouble(val as double);
        break;
      case 'float':
      case 'double':
        if (val == null) {
          writeDouble(0.0);
          break;
        }
        writeDouble(val as double);
        break;
      case 'java.util.Date':
        if (val == null) {
          writeNull();
          break;
        }
        writeUTCDate(val as DateTime);
        break;
      case 'java.lang.Object':
        if (val == null) {
          writeNull();
          break;
        }
        if (val is int) {
          writeLong(val);
        } else if (val is String) {
          writeString(val);
        } else if (val is double) {
          writeDouble(val);
        } else if (val is bool) {
          writeBoolean(val);
        } else if (val is DateTime) {
          writeUTCDate(val);
        } else if (val is Map) {
          writeMapValue(val, (k, v) { writeObjectValue('java.lang.Object', k, null);writeObjectValue('java.lang.Object', v, null);});
        } else if (val is List) {
          writeListValue(val, (e) => writeObjectValue('java.lang.Object', e, null));
        } else if (val is Set) {
          writeSetValue(val, (e) => writeObjectValue('java.lang.Object', e, null));
        } else if (val is HessianObjectBase) {
          writeObject(val, genericMap);
        } else {
          throw new Exception('Object is not supported ${val.runtimeType}');
        }
        break;
      default:
        if (val == null) {
          writeNull();
          break;
        }
        if (val is HessianObjectBase) {
          writeObject(val, genericMap);
        } else {
          throw new Exception('Class must implements HessianObjectBase');
        }
        break;
    }
  }

  Uint8List toBuffer() {
    if (_chunkBlocks.isEmpty) {
      return null;
    }

    if (_chunkBlocks.length == 1) {
      if (_chunkBlocks[0].bytesLen == _chunkBlocks[0].uint8list.length) {
        return _chunkBlocks[0].uint8list;
      }
      return _chunkBlocks[0].uint8list.buffer.asUint8List(0, _chunkBlocks[0].bytesLen);
    }

    _totalBytes = 0;
    _chunkBlocks.forEach((chunk) {
      _totalBytes += chunk.bytesLen;
    });

    int chunkOffset = 0;
    Uint8List data = new Uint8List(_totalBytes);
    _chunkBlocks.forEach((chunk) {
      data.setRange(chunkOffset, chunkOffset + chunk.bytesLen, chunk.uint8list);
      chunkOffset += chunk.bytesLen;
    });
    return data;
  }

}

