class IdentityIntMap {
  static int NULL = 0xdeadbeef; // Integer.MIN_VALUE + 1;

  List<dynamic> _keys;
  List<int> _values;

  int _size;
  int _prime;

  IdentityIntMap(int capacity) {
    _keys = new List(capacity);
    _values = new List<int>(capacity);

    _prime = getBiggestPrime(_keys.length);
    _size = 0;
  }

  void clear() {
    final List keys = _keys;
    final List<int> values = _values;

    for (int i = keys.length - 1; i >= 0; i--) {
      keys[i] = null;
      values[i] = 0;
    }

    _size = 0;
  }

  /**
   * Returns the current number of entries in the map.
   */
  int size() {
    return _size;
  }

  /**
   * Puts a new value in the property table with the appropriate flags
   */
  int get(dynamic key) {
    int prime = _prime;
//    int hash = System.identityHashCode(key) % prime;
    int hash = key.hashCode % prime;

    final List keys = _keys;

    while (true) {
      Object mapKey = keys[hash];

      if (mapKey == null)
        return NULL;
      else if (mapKey == key)
        return _values[hash];

      hash = (hash + 1) % prime;
    }
  }

  /**
   * Puts a new value in the property table with the appropriate flags
   */
  int put(Object key, int value, bool isReplace) {
    int prime = _prime;
//    int hash = Math.abs(System.identityHashCode(key) % prime);
    int hash = key.hashCode % prime;

    List keys = _keys;

    while (true) {
      Object testKey = keys[hash];

      if (testKey == null) {
        keys[hash] = key;
        _values[hash] = value;

        _size++;

        if (keys.length <= 4 * _size)
          resize(4 * keys.length);

        return value;
      } else if (key != testKey) {
        hash = (hash + 1) % prime;

        continue;
      } else if (isReplace) {
        int old = _values[hash];

        _values[hash] = value;

        return old;
      } else {
        return _values[hash];
      }
    }
  }

  /**
   * Removes a value in the property table.
   */
  void remove(Object key) {
    if (put(key, NULL, true) != NULL) {
      _size--;
    }
  }

  /**
   * Expands the property table
   */
  void resize(int newSize) {
    List keys = _keys;
    List<int> values = _values;

    _keys = new List(newSize);
    _values = new List<int>(newSize);
    _size = 0;

    _prime = getBiggestPrime(_keys.length);

    for (int i = keys.length - 1; i >= 0; i--) {
      Object key = keys[i];
      int value = values[i];

      if (key != null && value != NULL) {
        put(key, value, true);
      }
    }
  }

  String toString() {
    StringBuffer sbuf = new StringBuffer();

    sbuf.write("IntMap[");
    bool isFirst = true;

    for (int i = 0; i <= _keys.length; i++) {
      if (_keys[i] != null) {
        if (!isFirst)
          sbuf.write(", ");

        isFirst = false;
        sbuf.write(_keys[i]);
        sbuf.write(":");
        sbuf.write(_values[i]);
      }
    }
    sbuf.write("]");

    return sbuf.toString();
  }

  static final List<int> PRIMES = [
    1, /* 1<< 0 = 1 */
    2, /* 1<< 1 = 2 */
    3, /* 1<< 2 = 4 */
    7, /* 1<< 3 = 8 */
    13, /* 1<< 4 = 16 */
    31, /* 1<< 5 = 32 */
    61, /* 1<< 6 = 64 */
    127, /* 1<< 7 = 128 */
    251, /* 1<< 8 = 256 */
    509, /* 1<< 9 = 512 */
    1021, /* 1<<10 = 1024 */
    2039, /* 1<<11 = 2048 */
    4093, /* 1<<12 = 4096 */
    8191, /* 1<<13 = 8192 */
    16381, /* 1<<14 = 16384 */
    32749, /* 1<<15 = 32768 */
    65521, /* 1<<16 = 65536 */
    131071, /* 1<<17 = 131072 */
    262139, /* 1<<18 = 262144 */
    524287, /* 1<<19 = 524288 */
    1048573, /* 1<<20 = 1048576 */
    2097143, /* 1<<21 = 2097152 */
    4194301, /* 1<<22 = 4194304 */
    8388593, /* 1<<23 = 8388608 */
    16777213, /* 1<<24 = 16777216 */
    33554393, /* 1<<25 = 33554432 */
    67108859, /* 1<<26 = 67108864 */
    134217689, /* 1<<27 = 134217728 */
    268435399, /* 1<<28 = 268435456 */
  ];

  static int getBiggestPrime(int value) {
    for (int i = PRIMES.length - 1; i >= 0; i--) {
      if (PRIMES[i] <= value)
        return PRIMES[i];
    }

    return 2;
  }
}
