import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'dart:isolate';
import 'dart:typed_data';

import 'dart-ext:dart_hsfcpp_extension';
import 'package:saker/framework/environment/env_base.dart';
import 'package:saker/framework/hsf/hsf_info.dart';
import 'package:saker/framework/hsf/hsf_rpc_base.dart';
import 'package:saker/framework/log/log.dart';

void hsf_core_provider_init() native "hsf_core_provider_init";

void hsf_core_init(String snapshotPath, String logPath, int logLevel, String appName, int ioThreadCnt, int workerThreadCnt)  native "hsf_core_init";
void hsf_core_wait() native "hsf_core_wait";
void hsf_core_stop() native "hsf_core_stop";
void hsf_core_destroy() native "hsf_core_destroy";
int hsf_create_consumer(String consumerName, String version, String group, String aliasName, String timeOutConfig) native "hsf_create_consumer";
//int rpc_cli_eagleeye_end(String trace_id,int success_flag,int resp_size,String rpc_id,int server_idx) native "rpc_cli_eagleeye_end" ;//success_flag:0 success ,1 faled.
//int rpc_cli_eagleeye_start(String http_headers) native "rpc_cli_eagleeye_start";

List hsf_get_response(int resultAddr) native "hsf_get_response";
List hsf_release_response(int resultAddr) native "hsf_release_response";
List hsf_get_request(int reqAddr) native "hsf_get_request";
List register_worker_isolate(SendPort port, SendPort hsfProviderPool, bool onlyConsumer) native "register_worker_isolate";
int hsf_consumer_set_timeout(int consumerAddress, String methodName,int timeOut) native "hsf_consumer_set_timeout";
int hsf_send_respone(int contextAddress,String errorMsg,  Uint8List response) native "hsf_send_respone";
int hsf_create_provider(serviceJsonString) native "hsf_create_provider";
int http_eagleeye_end(String traceId,int successFlag,int respSize) native "http_eagleeye_end" ;//success_flag:0 success ,1 faled.
int http_eagleeye_start(String httpHeaders) native "http_eagleeye_start";

int hsf_consumer_invoke_async_1_param(int consumerAddress, String methodName, int eventId,String traceId, String rpcId, String invokeIp,
    String eagleeyeCtxHessiaanJson, int resultAddress, int flags, String rpcContext, String paramTypes, Uint8List params) native "hsf_consumer_invoke_async_1_param";

int hsf_consumer_invoke_async(int consumerAddress, String methodName, int eventId, String traceId, String rpcId, String invokeIp,
    String eagleeyeCtxHessiaanJson,  int resultAddress, int flags, String rpcContext,  List<String> paramTypes, List<Uint8List> params) native "hsf_consumer_invoke_async";

int hsf_consumer_invoke_async_by_json(int consumerddress, String methodName, String paramJSON, int eventId,
    String traceId, String rpcId, String invokeIp, String eagleeyeCtxHessiaanJson, int result_address,int flag, String rpcContext) native "hsf_consumer_invoke_async_by_json";

int hsf_consumer_invoke(int consumerAddress, String methodName, int eventId, String traceId, String rpcId, String invokeIp, int resultAddress,
    String paramStr,  List<String> paramTypes, List<Uint8List> params, {String eagleeyeCtxHessiaanJson:"", int invokeType:hsfInvokeTypeNative, int encodeType:hsfEncodeTypeHessianJson, String rpcContext:""}) {

  int flags = hsfInvokeTypeMesh == invokeType ? 0x1 : 0;

  //hessian直传
  if(encodeType == hsfEncodeTypeHessian){
    if(params.length == 1){
      return hsf_consumer_invoke_async_1_param(consumerAddress, methodName, eventId, traceId, rpcId, invokeIp, eagleeyeCtxHessiaanJson, resultAddress, flags, rpcContext, paramTypes[0], params[0]);
    }

    return hsf_consumer_invoke_async(consumerAddress, methodName, eventId, traceId, rpcId, invokeIp, eagleeyeCtxHessiaanJson, resultAddress, flags, rpcContext, paramTypes, params);
  }
  //hessian json
  return hsf_consumer_invoke_async_by_json(consumerAddress, methodName, paramStr,  eventId, traceId, rpcId, invokeIp, eagleeyeCtxHessiaanJson, resultAddress, flags, rpcContext);
}

int hsfResultAddress = 0;
int hsfRequestAddress = 0;
void register_isolate(SendPort sendPort, SendPort hsfProviderPool){
  List<int> resList = register_worker_isolate(sendPort, hsfProviderPool, EnvFramework.isHotReload)?.cast<int>();

  if(resList != null && resList.length == 3 && resList[0] == 0){
    hsfResultAddress = resList[1];
    hsfRequestAddress = resList[2];
    MyLogger.getLogger().error("regiester worker isolate success. result addr:$hsfResultAddress, req address:$hsfRequestAddress");
  } else {
    MyLogger.getLogger().error("regiester worker isolate failed.");
  }
}

class HttpPool {
  Map<String, HttpClient> httpPool = {};

  HttpClient getConnection(String host) {
    if (httpPool.containsKey(host)) {
      return httpPool[host];
    }
    HttpClient _httpClient = new HttpClient();
    httpPool[host] = _httpClient;
    return _httpClient;
  }
}

HttpPool httpPool = new HttpPool();

Future<String> hsf_http_invoke(HsfInfo hsfInfo, String methodName, String dartServiceMethod, String paramJSON, Duration timeout, String traceId, String rpcId) async {

  HttpClient httpClient = httpPool.getConnection(hsfInfo.httpHost);//new HttpClient();
  HttpClientRequest request = await httpClient.post(hsfInfo.httpHost, hsfInfo.httpPort??12220, '/${hsfInfo.consumerName}/${hsfInfo.version}/${methodName}')
      .timeout(timeout).catchError((e) => MyLogger.getLogger().error('HSF HTTP REQ ERROR:$dartServiceMethod ${e}'));
  if (request == null) {
    return null;
  }

  request.headers.add('Content-Type', 'application/fastjson');
  request.headers.add('Http-Rpc-Type', 'JsonContent');
  request.headers.add('Http-Rpc-Group', hsfInfo.group);
  request.headers.add('EagleEyeX-traceid', traceId);
  request.headers.add('EagleEyeX-rpcid', rpcId);

  request.write(paramJSON);

  HttpClientResponse response = await request.close()
      .catchError((e) => MyLogger.getLogger().error('HSF HTTP RES ERROR:$dartServiceMethod ${e}'));
  if (response == null) {
    return null;
  }

  String resJson = await response.transform(utf8.decoder).join()
      .catchError((e) => MyLogger.getLogger().error('HSF HTTP JOIN ERROR:$dartServiceMethod ${e}'));
  return resJson;
}

//Future<Map> hsf_http_invoke(HsfInfo hsfInfo,
//    String methodName,
//    String paramJSON,
//    Duration timeout,
//    String traceId,
//    String rpcId) async {
//  HttpClient httpClient = httpPool.getConnection(hsfInfo.httpHost);//new HttpClient();
//  HttpClientRequest request = await httpClient.post(hsfInfo.httpHost, hsfInfo.httpPort??12220, '/${hsfInfo.consumerName}/${hsfInfo.version}/${methodName}')
//      .catchError((e) => print(e)).timeout(timeout)
//    ..headers.add('Content-Type', 'application/json')
//    ..headers.add('Http-Rpc-Type', 'JsonContent')
//    ..headers.add('eagleeye-traceid', traceId)
//    ..headers.add('eagleeye-rpcid', rpcId)
//    ..write(paramJSON);
//  if (request == null) {
//    return {'code':HSF_ERROR_SERVER};
//  }
//
//  HttpClientResponse response = await request.close().catchError((e) => print(e));
//  if (response == null) {
//    return {'code':HSF_ERROR_SERVER};
//  }
//
//  String resJson = await response.transform(utf8.decoder).join().catchError((e) => print(e));
//  if (resJson == null) {
//    return {'code':HSF_ERROR_SERVER};
//  }
//
//  try {
//    Map resultMap = json.decode(resJson);
//    return resultMap;
//  } catch (e) {
//    return {'code':HSF_ERROR_SERVER};
//  }
//}