import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'package:http/http.dart' as http;
import 'package:saker/framework/environment/env_base.dart';
import 'package:saker/framework/log/dart_spdlog_extension.dart';
import 'package:saker/framework/trace/trace_info.dart';



class MyLogger {
//  static String rootPath = null;
//  static LogLevel log_level = LogLevel.debug;
  static Map<String, Logger> loggers = {};
  static String isolateName = "SYSTEM";

  static void initLogger(String rootPath, String logLevel, String projectName, String tag) {
    isolateName = tag;

    spdlog_init();

    addNewLogger('default', '${EnvFramework.rootPath}/logs/${projectName}.log', logLevel, LogFileType.daily_log);
    addNewLogger('caiNiaoMonitor', '${EnvFramework.rootPath}/logs/${EnvFramework.monitorLogFile}', "warn", LogFileType.rotate_log, rotate_file_size: 2048*1024*1024, rotate_file_cnt: 10);
    addNewLogger('dartVM', '/home/admin/logs/eagleeye/stat-eagleeye-jvm.log', "warn", LogFileType.rotate_log, rotate_file_size: 64*1024*1024, rotate_file_cnt: 2);
    addNewLogger('flowLimit', '${EnvFramework.rootPath}/logs/flowLimit.log', "warn", LogFileType.rotate_log, rotate_file_size: 64*1024*1024, rotate_file_cnt: 2);
    addNewLogger('trafficGuard', '${EnvFramework.rootPath}/logs/trafficGuard.log', logLevel, LogFileType.rotate_log, rotate_file_size: 64*1024*1024, rotate_file_cnt: 2);

  }

  //增加新的logger
  static void addNewLogger(String name, String path, String logLevel, LogFileType logType,{int rotate_file_size:512*1024*1024, int rotate_file_cnt: 5, Format format}) {
    loggers[name] = new Logger(name, logType, getLogLevel(logLevel), path, max_rotate_file_size: rotate_file_size, max_rotate_file_cnt:rotate_file_cnt, format: format);
  }

//  static void close() {
//    spdlog_close();
//  }

  static Logger getLogger({String name}) {
    if (name != null && name.isNotEmpty && loggers.containsKey(name)){
      return loggers[name];
    }
    return loggers["default"];
  }

  static Logger getDebugLogger({String name}) {
    Logger logger = getLogger(name: name);
    if (logger.levelDebug) {
      return logger;
    }
    return null;
  }

  static bool logRegistered(String name){
    if(name != null && name.isNotEmpty){
      return loggers.containsKey(name);
    }
    return false;
  }
}

Future log_write(String name,int logger_addr ,int level, String levelName, String msg) async {
  TraceInfo traceInfo = TraceInfo.getTraceInfo();
  DateTime now = new DateTime.now();

  if(name == "caiNiaoMonitor" || name == "flowLimit" || name == "userTrace" || name.startsWith('faas_')){
    spdlog_write(logger_addr, level, "${now.toString()}|${msg}");
  }else if(name == "dartVM"){
    spdlog_write(logger_addr, level, "${msg}");
  }else {
    if (traceInfo != null) {
      spdlog_write(logger_addr, level, "${now.toString()} $levelName [${MyLogger.isolateName}] [${traceInfo?.eagleeyeTraceid}] [${traceInfo?.eagleeyeRpcid}] ${msg}");
      traceInfo.debugLogBuffer?.writeln("${now.toString()} $levelName [${MyLogger.isolateName}] [${traceInfo?.eagleeyeTraceid}] [${traceInfo?.eagleeyeRpcid}] ${msg}");
    } else {
      spdlog_write(logger_addr, level, "${now.toString()} $levelName [${MyLogger.isolateName}] ${msg}");
    }
  }

  try{
    if(name == "default" && EnvFramework.logServerUrl.isNotEmpty){
      var url = EnvFramework.logServerUrl;
      String logMsg = jsonEncode({"isolate": "${MyLogger.isolateName}","logtime":now.microsecondsSinceEpoch,
        "logMsg": "${now.toString()} $levelName [${MyLogger.isolateName}] [${traceInfo?.eagleeyeTraceid}] [${traceInfo?.eagleeyeRpcid}] ${msg}"});
//      String logMsg = jsonEncode({"isolate": "${MyLogger.isolateName}","logtime":now.microsecondsSinceEpoch,
//        "logMsg": "${now.toString()} "});

//    HttpClientRequest request = await Logger.logclient.post(AppConfig.dailyLogServer, 80, "/dartserverLog")
//        .catchError((e) => print(e))
//      ..headers.add('Content-Type', 'application/json')
//      ..headers.contentLength = logMsg.length
//      ..write(logMsg);
//      await request.close().catchError((e) => print(e));

      await http.post(url, headers: {"Content-Type": "application/json"}, body: logMsg )
          .then((response) {
      }).catchError((e){
      });
    }
  }catch(e){
    print("log_write exception:$e");
  }
}

typedef Format = String Function(String msg, String logLevelName);

class Logger {
  int _log_level = 0;
  String _log_name;
  int _logger_addr = 0;
  Format _format;
  static final HttpClient logclient = new HttpClient();

  Logger(String log_name, LogFileType log_type, LogLevel log_level,
      String log_path,{int max_rotate_file_size:1024*1024*512, int max_rotate_file_cnt: 5, Format format}) {
    _log_level = log_level.index;
    _log_name = log_name;
    _format = format;
    _logger_addr =  register_logger(log_name, log_type.index, log_path, log_level.index, max_rotate_file_size,max_rotate_file_cnt);
  }

  bool get levelDebug => _log_level <= LogLevel.debug.index;

  trace(String msg){
    if (_log_level <= LogLevel.trace.index) {
      if(_format != null){
        msg = _format(msg, "TRACE");
      }
      if(msg != null && msg.length > 0) {
        log_write(_log_name, _logger_addr, LogLevel.trace.index, "TRACE", msg);
      }
    }
  }

  debug(String msg) {
    if (_log_level <= LogLevel.debug.index) {
      if(_format != null){
        msg = _format(msg, "DEBUG");
      }
      if(msg != null && msg.length > 0) {
        log_write(_log_name, _logger_addr, LogLevel.debug.index, "DEBUG", msg);
      }
    }
  }

  info(String msg) {
    if (_log_level <= LogLevel.info.index) {
      if(_format != null){
        msg = _format(msg,"INFO");
      }
      if(msg != null && msg.length > 0) {
        log_write(_log_name, _logger_addr, LogLevel.info.index, "INFO", msg);
      }
    }
  }

  error(String msg) {
    if (_log_level <= LogLevel.err.index) {
      if(_format != null){
        msg = _format(msg,"ERROR");
      }
      if(msg != null && msg.length > 0) {
        log_write(_log_name, _logger_addr, LogLevel.err.index, "ERROR", msg);
      }
    }
  }

  warn(String msg) {
    if (_log_level <= LogLevel.warn.index) {
      if(_format != null){
        msg = _format(msg,"WARN");
      }
      if(msg != null && msg.length > 0) {
        log_write(_log_name, _logger_addr, LogLevel.warn.index, "WARN", msg);
      }
    }
  }

}