import 'dart:isolate';

import 'package:saker/framework/environment/env_base.dart';
import 'package:saker/framework/event/event_pool.dart';
import 'package:saker/framework/event/event_pool_base.dart';
import 'package:saker/framework/event/notice_type.dart';
import 'package:saker/framework/hsf/hsf_provider_base.dart';
import 'package:saker/framework/log/log.dart';
import 'package:saker/framework/services/service_dispatch.dart';
import 'package:saker/framework/services/service_var.dart';

void biz_isolate_notice_callback(dynamic messages) {
  if (messages is! List) {
    return;
  }

  int nt = messages[0] as int;
  SendPort sendPort = messages[1] as SendPort;
  int eventId = messages[2] as int;
//  var body = messages[3];

  if (nt == NoticeType.BizHttpReq) {
    try {
      List body = messages[3] as List;

      processDispatch(body[0] as String, body[1] as Map, (body[2] as Map)?.cast<String, String>(), (body[3] as Map)?.cast<String, String>()).then((value) {
        sendPort.send([EventType_Any, eventId, EventStatus_OK, value]);
      }).catchError((e) {
        MyLogger.getLogger().error("Biz isolate error async:$e");
        sendPort.send([EventType_Any, eventId, EventStatus_OK, null]);
      });
    } catch (e) {
      MyLogger.getLogger().error("Biz isolate error catch:$e");
      sendPort.send([EventType_Any, eventId, EventStatus_OK, null]);
    }
  } else if (nt == NoticeType.BizHsfReq) {
    try {
      List body = messages[3] as List;

      HsfProviderBase.processHsfDispatch(body, true).then((value) {
        sendPort.send([EventType_Any, eventId, EventStatus_OK, value]);
      }).catchError((e) {
        MyLogger.getLogger().error("Biz hsf isolate error async:$e");
        sendPort.send([EventType_Any, eventId, EventStatus_OK, null]);
      });
    } catch (e) {
      MyLogger.getLogger().error("Biz hsf isolate error catch:$e");
      sendPort.send([EventType_Any, eventId, EventStatus_OK, null]);
    }
  } else if (nt == NoticeType.BizIsolateClose) {
    MyLogger.getLogger().info("Biz isolate has been closed");

    sendPort.send([EventType_Any, eventId, EventStatus_OK, 0]);

    EnvBase.getEnv().releaseIsolate();

    Isolate.current.kill();
  } else {
    MyLogger.getLogger().warn("Biz isolate notice discard: ${messages}");
  }
}

void startBizApp(List<dynamic> messages) async {
  SendPort sendPort = messages[1] as SendPort;
  int eventId = messages[2] as int;

  serviceEntries = EnvBase.getEnv().getServiceEntries();
  EventPool.notice_cb = biz_isolate_notice_callback;

  EnvFramework.isHotReload = true;

  EnvBase.getEnv().initIsolate();

  sendPort.send([EventType_Any, eventId, EventStatus_OK, EventPool.sendPort]);
}
