import 'dart:async';

import 'package:saker/framework/environment/env_base.dart';
import 'package:saker/framework/log/log.dart';
import 'package:vm_service_lib/vm_service_lib.dart';
import 'package:vm_service_lib/vm_service_lib_io.dart';

Future<bool> reloadSource(String isolateId, String dillFile) async {
  final String host = EnvFramework.observeAddr;
  final int port = EnvFramework.observePort;

  var serviceClient = await vmServiceConnect(host, port).catchError((e) {
    MyLogger.getLogger().error('ReloadSource:socket connected:$host:$port, err=$e');
  });

  MyLogger.getLogger().info('ReloadSource:${isolateId}');
  ReloadReport report = await serviceClient.reloadSources(isolateId, rootLibUri: dillFile).catchError((e) {
    MyLogger.getLogger().error('ReloadSource:error, err=$e');
  });
  if (report.success != true) {
    MyLogger.getLogger().error('ReloadSource:error, err=${report.json}');
  }
  if (report == null) {
    return false;
  }
  return report.success;
}

Future<String> findIsolateIdByName(String isolateName) async {
  final String host = EnvFramework.observeAddr;
  final int port = EnvFramework.observePort;

  var serviceClient = await vmServiceConnect(host, port).catchError((e) {
    MyLogger.getLogger().error('findIsolateIdByName:socket connected:$host:$port, err=$e');
  });
  var report = await serviceClient.getVM();
  for (IsolateRef ref in report.isolates) {
    if (ref.name.contains(isolateName)) {
      return ref.id;
    }
  }
  return null;
}

Future<List<String>> findAllHttpIsolateIds({name = 'worker_isolate_entry'}) async {
  final String host = EnvFramework.observeAddr;
  final int port = EnvFramework.observePort;

  var serviceClient = await vmServiceConnect(host, port).catchError((e) {
    MyLogger.getLogger().error('findAllHttpIsolateIds:socket connected:$host:$port, err=$e');
  });
  var report = await serviceClient.getVM();
  return report.isolates
      .where((IsolateRef ref)=>ref.name.contains(name))
      .map((IsolateRef ref)=>ref.id)
      .toList();
}
