import 'dart:convert';
import 'dart:isolate';

import 'package:saker/framework/environment/env_base.dart';
import 'package:saker/framework/event/event_pool.dart';
import 'package:saker/framework/event/event_pool_base.dart';
import 'package:saker/framework/event/notice_type.dart';
import 'package:saker/framework/log/log.dart';
import 'package:saker/framework/reloader/reloader_server.dart';
import 'package:saker/framework/stat/dump_dart_vm_infos.dart';

ReloadServer mainServer;

void main_notice_callback(dynamic messages) {
  MyLogger.getDebugLogger()?.debug(json.encode("main_notice_callback :${messages}"));
  if (messages is! List) {
    return;
  }
  var nt = messages[0] as int;

  if (nt == NoticeType.HttpOffline) {
    MyLogger.getLogger().info("Main notice offline");
    if (mainServer != null) {
      mainServer.broadcastHttpIsolate(nt, 0);
      mainServer.hsfOffline();
    }
  } else if (nt == NoticeType.WorkerIsolateOffline) {
    MyLogger.getLogger().info("Main notice isolate offline");
    if (mainServer != null) {
      mainServer.broadcastHttpIsolate(NoticeType.HttpOffline, 0);
      mainServer.hsfOffline();
      mainServer.checkAllIsolateReady();
    }
  } else if (nt == NoticeType.HttpOnline) {
    MyLogger.getLogger().info("Main notice online");
    if (mainServer != null) {
      mainServer.broadcastHttpIsolate(nt, 1);
      mainServer.hsfOnline();
    }
  } else if (nt == NoticeType.HsfProviderMap) {
    MyLogger.getLogger().info("Main notice hsf provider map update");
    if (mainServer != null) {
      try {
        Map<dynamic, dynamic> m = json.decode(messages[2] as String) as Map<dynamic, dynamic>;
        mainServer.hsfProviderMap.addAll(m.cast<String, dynamic>());
      } catch (e, stack) {
        MyLogger.getLogger().info("Main notice hsf provider map update, error=${e}, stack=${stack}");
      }
    }
  } else if (nt == NoticeType.BizHotReload) {
    if (mainServer != null) {
      SendPort sendPort = messages[1] as SendPort;
      int eventId = messages[2] as int;
      String uTag = messages[3] as String;
      String compileType = messages[4] as String;
      mainServer.reload(uTag, compileType: compileType).then((strRes) {
        sendPort.send([EventType_Any, eventId, EventStatus_OK, strRes]);
      }).catchError((e) {
        MyLogger.getLogger().warn("Main notice hot reload error: $e");
        sendPort.send([EventType_Any, eventId, EventStatus_OK, 'Main hot reload error']);
      });
    }
  } else {
    MyLogger.getLogger().warn("Main notice discard: ${messages}");
  }
}

void startMainApp() async {
  EventPool.notice_cb = main_notice_callback;

  mainServer = new ReloadServer(EnvBase.getEnv());
  mainServer.start();

  dumpDartVMInfos();
}

