import 'dart:async';

import 'package:saker/framework/jsonserializer/json_utils.dart';
import 'package:saker/framework/jsonserializer/proto.dart';
import 'package:saker/framework/services/service_result.dart';
import 'package:saker/framework/services/mtop_context.dart';
import 'package:saker/framework/stat/service_flow_limit.dart';

export 'package:saker/framework/services/mtop_context.dart';

typedef ApiRequestBase ServiceRequestParser(final Map<dynamic, dynamic> data);

abstract class ServiceBase<REQ extends ApiRequestBase, RES extends ApiResponseBase> extends ServiceFlowLimit {
  ServiceRequestParser serviceRequestParser;

  void init(){}

  void destroy(){}

  Future<ServiceResult> process(final REQ req, final MtopContext context);

  Future<ServiceResult> processMtop(final Map<dynamic, dynamic> data,
      final MtopContext context) {
    REQ req = serviceRequestParser(data) as REQ;
    return process(req, context);
  }

  ServiceResult buildSuccResult(dynamic data) {
    ServiceResult r = new ServiceResult()
      ..errcode = 'SUCCESS'
      ..errmsg = '接口调用成功';

    if (data is RES) {
      r.data = (JsonUtil.translateMTOP(data?.toJson())) ?? {};
    } else {
      r.data = (JsonUtil.translateMTOP(data));
    }
    if (r.data != null && r.data is Map) {
      r.data['serverTime'] = getServerTime();
    }
    return r;
  }

  ServiceResult buildFailResult(String errcode, String errmsg) {
    return new ServiceResult()
      ..data = {}
      ..errcode = errcode
      ..errmsg = errmsg;
  }

  String getServerTime() {
    String t = new DateTime.now().toString();
    return t.substring(0, t.lastIndexOf('.'));
  }

  static MtopContext getZoneContext() {
    return (Zone.current[#MtopContext] as MtopContext) ?? MtopContext.EMPTY;
  }
}