import 'dart:convert';
import 'dart:io';
import 'dart:async';
import 'dart:isolate';

import 'package:saker/framework/environment/env_base.dart';
import 'package:saker/framework/services/service_process.dart';
import 'package:saker/framework/log/log.dart';
import 'package:saker/framework/services/service_server.dart';

HttpServer _httpServer = null;

final Duration HTTP_CLOSE_TIMEOUT = new Duration(milliseconds: 500);

class HttpServerService extends AbstractServer{

  @override
  String get serverKind => HTTP;

  @override
  Future close({String type, SendPort sendPort, int eventId})  {
    return _httpServer.close(force: true).timeout(HTTP_CLOSE_TIMEOUT);
  }

  @override
  Future start({List<dynamic> param}) {
   return  HttpServer.bind(EnvFramework.httpAddr, EnvFramework.httpPort, shared: true).then(
           (server) {
              _httpServer = server;
              StreamSubscription subscription;
              subscription = server.listen(
                      (HttpRequest request) {
                        processHttp(request)
                            .catchError(
                                    (e,stack) {
                                        MyLogger.getLogger().error('HTTP Server Error:${e}\n${stack}');
                                        request.response..statusCode = HttpStatus.badRequest
                                          ..write('HTTP Server error "${request.uri.path}"')
                                          ..close().catchError((e) => MyLogger.getLogger().error('_writeServerError:$e'));
                                      }
                            );
                      },
                      onDone: () {
                        subscription.cancel();
                      }
              );
            }
    );
  }

  @override
  Future init(List<dynamic> param){
    //noting to do
  }

  @override
  void offline({SendPort sendPort, int eventId, int offline}) {
    //nothing to do
  }

  @override
  void shutdown({SendPort sendPort, int eventId}) {
    //nothing to do
  }

}
