import 'dart:convert';
import 'dart:io';
import 'package:saker/common/utils/pretty_json.dart';
import 'package:saker/common/utils/uuid.dart';

class ServiceTest {
  static final String appKey = '4242';

  static String mapToQuery(Map<String, String> map, {Encoding encoding}) {
    var pairs = <List<String>>[];

    map.forEach((key, value) {
      if (value != null) {
        pairs.add([
          Uri.encodeQueryComponent(key, encoding: encoding),
          Uri.encodeQueryComponent(value, encoding: encoding)
        ]);
      }
    });
    return pairs.map((pair) => "${pair[0]}=${pair[1]}").join("&");
  }

  static void runTest(String url, String isolateName, String env, Map<String, dynamic> data) async {
    String uid = null;
    String nick = null;
    if (data.containsKey('_nick')) {
      nick = data['_nick'] as String;
      data.remove('_nick');
    }
    if (data.containsKey('_uid')) {
      uid = data['_uid'] as String;
      data.remove('_uid');
    }

    String uuid = UuidUtil.getUuid();
    Map<String, String> headers = {
      'user-agent': 'Dart-Tester',
      'x-request-from': 'mtop',
      'x-mg-userid': '${uid}',
      'content-type': 'application/x-www-form-urlencoded',
      'x-mg-ip': '127.0.0.1',
      'x-mg-v': '1.0',
      'x-mg-api': 'mtop.taobao.idle.dart.testapi',
      'x-mg-alipayid': 'non-value',
      'x-mg-sid': 'non-value',
      'x-mg-nativeproxyh5': 'false',
      'x-mg-ttid': '11320%40taobao_android_5.0.0',
      'x-mg-appkey': '${appKey}',
      'eagleeye-userdata': 'i=8d59e34d&u=${uid}&bt=62e77a2460&dpath_env=${isolateName}',
      'x-mg-nick': '${nick}',
      'eagleeye-rpcid': '0.1',
      'x-remote-ip': '127.0.0.1',
      'x-mg-umid': '',
      'eagleeye-traceid': '${uuid}',
      'x-mg-utdid': 'Degarde'
    };

    Map<String, String> context = {
      'data': json.encode(data),
      'userId': uid,
      'nick': nick,
      'platform': 'android',
      'deviceId': 'non-value',
      'ttid': '11320%40taobao_android_5.0.0',
      'appkey': appKey
    };

    if (url.startsWith('redirect-hosts=')) {
      var urls = url.split('|');
      headers['redirect-hosts'] = urls[0].substring(15);
      url = urls[1];
    }

    Stopwatch stopwatch = new Stopwatch()..start();

    HttpClient _httpClient = new HttpClient();
    HttpClientRequest request = await _httpClient.postUrl(Uri.tryParse(url))
        .timeout(new Duration(seconds: 5))
        .catchError((e) => print('$e'));

    if (request == null) {
      return;
    }

    headers.forEach((k, v) {
      if (v != null) {
        request.headers.add(k, v);
      }
    });
    request.write(mapToQuery(context, encoding: utf8));

    HttpClientResponse response = await request.close().catchError((e) => print('$e'));

    if (response == null) {
      return;
    }

    stopwatch.stop();
    print('Http Code:${response.statusCode}, Duration=${stopwatch.elapsedMilliseconds}ms');

    if (response.statusCode == 200) {
      String body = await response.transform(utf8.decoder).join();
      Map m = json.decode(body) as Map;
      if (m.containsKey('ret')) {
        print('\n-----------------ret------------------');
        print(m['ret']);
      }
      if (m.containsKey('data')) {
        print('\n-----------------data-----------------');
        print(PrettyJSON.encode(m['data']));
      }
      if (m.containsKey('log')) {
        print('\n-----------------log------------------');
        print(m['log']);
      }
    } else {
      print('\n---------------error-body----------------');
      print(response.toString());
    }

    _httpClient.close(force: true);
  }
}