import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:saker/common/net/ip_address.dart';
import 'package:saker/framework/environment/env_base.dart';
import 'package:saker/framework/log/log.dart';
import 'package:saker/framework/log/log_type.dart';
import 'package:saker/framework/stat/service_monitor.dart';
import 'package:saker/common/utils/date_utils.dart';
import 'package:vm_service_lib/vm_service_lib.dart';
import 'package:vm_service_lib/vm_service_lib_io.dart';


int dartPid = 0;

T getJsonVal<T>(Map m, String path) {
  List<String> paths = path.split('/');

  Map m1 = m;
  for(int i = 0; i < paths.length; i++) {
    if (i == paths.length - 1) {
      dynamic v = m1[paths[i]];
      if (v == null || v is! T) {
        MyLogger.getLogger().warn("dumpDartVMInfos getJsonVal return null: json: ${json.encode(m)}, path: ${paths[i]}, idx:$i");
        return null;
      }
      return v as T;
    } else {
      dynamic v = m1[paths[i]];
      if (v == null || v is! Map) {
        return null;
      }
      m1 = v as Map;
    }
  }

  MyLogger.getLogger().warn("dumpDartVMInfos getJsonVal return null: json: ${json.encode(m)}, path: ${path}");
  return null;
}

Future dumpDartVMInfos()async {
  final String host = EnvFramework.observeAddr;
  final int port = EnvFramework.observePort;

  var serviceClient = await vmServiceConnect(host, port, log: new vmServiceLog()).catchError((e) {
    MyLogger.getLogger().error('dumpDartVMInfos:socket connected:$host:$port, err=$e');
  });

  if (serviceClient == null) {
    return;
  }

  MyLogger.getLogger().info('dumpDartVMInfos:socket connected:$host:$port');

  serviceClient.onSend.listen((str) => MyLogger.getDebugLogger()?.trace('dumpDartVMInfos ---> ${str}'));
  serviceClient.onReceive.listen((str) => parseDartVMGCInfo(str));

  serviceClient.onGCEvent.listen((e) => MyLogger.getDebugLogger()?.trace('dumpDartVMInfos onGCEvent: ${e}'));
  serviceClient.streamListen('GC');

  VM vm = await serviceClient.getVM();
  dartPid = vm.pid;

  MyLogger.getLogger().debug("dumpDartVMInfos: dart pid:$dartPid host:${IPAddress.getHostIp()}");
  new Timer.periodic(new Duration(minutes: 1), (Timer timer) => flushDartVMInfo() );
  new Timer.periodic(new Duration(seconds: 6), (Timer timer) async => await getCpuUtility(dartPid) );

}


GCStatistics lastMinuteGCStatistic = new GCStatistics();
Map currGCInfos = {};
//简单处理：不以墙上时间的一分钟的60s作为上报的时间点，将定时器每分钟触发的时间作为上报点，将定时器触发的间隔内的gc信息都算作上一分钟的。简单粗暴有效
void flushDartVMInfo(){
  String timeMinute = DateTools.formatDateTime(new DateTime.now().subtract(new Duration(minutes: 1)), "yyyy-MM-dd HH:mm:00");
  GCStatistics curr = new GCStatistics();
  try {
    MyLogger.getDebugLogger()?.trace("dumpDartVMInfos flushDartVMInfo currGCInfo: ${json.encode(currGCInfos)}");
    //遍历每个isolate，统计信息
    currGCInfos.keys.forEach((isolateId){
      Map currGCInfo = currGCInfos[isolateId] as Map;
      //new gen
      curr.newSpace.gcCnt += getJsonVal<int>(currGCInfo, "params/event/new/collections")??0;
      curr.newSpace.gcTime += (getJsonVal<double>(currGCInfo, "params/event/new/time")??0.0) * 1000;
      curr.newSpace.used += getJsonVal<int>(currGCInfo, "params/event/new/used")??0;
      curr.newSpace.capacity += getJsonVal<int>(currGCInfo, "params/event/new/capacity")??0;
      //old gen
      curr.oldSpace.gcCnt += getJsonVal<int>(currGCInfo, "params/event/old/collections")??0;
      curr.oldSpace.gcTime += (getJsonVal<double>(currGCInfo, "params/event/old/time")??0.0) * 1000;
      curr.oldSpace.used += getJsonVal<int>(currGCInfo, "params/event/old/used")??0 ;
      curr.oldSpace.capacity += getJsonVal<int>(currGCInfo, "params/event/old/capacity")??0;
    });

    curr.newSpace.lastMinuteGCCnt = curr.newSpace.gcCnt - lastMinuteGCStatistic.newSpace.gcCnt;
    curr.newSpace.lastMinuteGCTime = curr.newSpace.gcTime - lastMinuteGCStatistic.newSpace.gcTime;
    curr.oldSpace.lastMinuteGCCnt = curr.oldSpace.gcCnt - lastMinuteGCStatistic.oldSpace.gcCnt;
    curr.oldSpace.lastMinuteGCTime = curr.oldSpace.gcTime - lastMinuteGCStatistic.oldSpace.gcTime;

    if(curr.newSpace.lastMinuteGCTime.toInt() < 0 || curr.newSpace.lastMinuteGCCnt < 0 || curr.oldSpace.lastMinuteGCCnt < 0 || curr.oldSpace.lastMinuteGCTime.toInt() < 0){
      MyLogger.getLogger().error("dumpDartVMInfos getJsonVal return null:${json.encode(currGCInfos)} "
          "\n newSpace: ${curr.newSpace.lastMinuteGCTime}, ${curr.newSpace.lastMinuteGCCnt}"
          "\n oldSpace: ${curr.oldSpace.lastMinuteGCCnt}, ${curr.oldSpace.lastMinuteGCTime}");
    }
    
    curr.newSpace.lastMinuteGCTime.toInt() < 0? curr.newSpace.lastMinuteGCTime = 0.0 : true;
    curr.newSpace.lastMinuteGCCnt < 0? curr.newSpace.lastMinuteGCCnt = 0: true;
    curr.oldSpace.lastMinuteGCCnt < 0? curr.oldSpace.lastMinuteGCCnt = 0 : true;
    curr.oldSpace.lastMinuteGCTime.toInt() < 0? curr.oldSpace.lastMinuteGCTime = 0.0 : true;

    lastMinuteGCStatistic = curr;

    MyLogger.getLogger(name: "dartVM").error("$timeMinute|3|$dartPid,gc,newGen|${curr.newSpace.lastMinuteGCCnt},${curr.newSpace.lastMinuteGCTime.toInt()},${curr.newSpace.gcCnt},${curr.newSpace.gcTime.toInt()}");
    MyLogger.getLogger(name: "dartVM").error("$timeMinute|3|$dartPid,gc,oldGen|${curr.oldSpace.lastMinuteGCCnt},${curr.oldSpace.lastMinuteGCTime.toInt()},${curr.oldSpace.gcCnt},${curr.oldSpace.gcTime.toInt()}");

    MyLogger.getLogger().trace("dumpDartVMInfos flush stat $timeMinute|3|$dartPid,gc,newGen|${curr.newSpace.lastMinuteGCCnt},${curr.newSpace.lastMinuteGCTime.toInt()},${curr.newSpace.gcCnt},${curr.newSpace.gcTime.toInt()}");
    MyLogger.getLogger().trace("dumpDartVMInfos flush stat $timeMinute|3|$dartPid,gc,oldGen|${curr.oldSpace.lastMinuteGCCnt},${curr.oldSpace.lastMinuteGCTime.toInt()},${curr.oldSpace.gcCnt},${curr.oldSpace.gcTime.toInt()}");


    MyLogger.getLogger(name: "dartVM").error("$timeMinute|3|$dartPid,memory,newGen|0,${curr.newSpace.used},${curr.newSpace.capacity},${curr.newSpace.capacity}");
    MyLogger.getLogger(name: "dartVM").error("$timeMinute|3|$dartPid,memory,oldGen|0,${curr.oldSpace.used},${curr.oldSpace.capacity},${curr.oldSpace.capacity}");
    MyLogger.getLogger().trace("dumpDartVMInfos:flush dart vm info:$timeMinute  for isolates: ${currGCInfos.keys.join(",")}");
  }catch(e, stack){
    MyLogger.getLogger().error(" dumpDartVMInfos: flushDartVMInfo exception:$e, $stack");
  }
}


/*
* "jsonrpc":"2.0",
        "method":"streamNotify",
        "params":{
            "streamId":"GC",
            "event":{
                "type":"Event",
                "kind":"GC",
                "isolate":{
                    "type":"@Isolate",
                    "fixedId":true,
                    "id":"isolates/382412975",
                    "name":"server.snapshot:http_server_isolate_entry()",
                    "number":"382412975"
                },
                "timestamp":1536721880223,
                "reason":"new space",
                "new":{
                    "type":"HeapSpace",
                    "name":"new",
                    "vmName":"Scavenger",
                    "collections":322,
                    "avgCollectionPeriodMillis":228677.875702,
                    "used":25440,
                    "capacity":33554432,
                    "external":160,
                    "time":0.410884
                },
                "old":{
                    "type":"HeapSpace",
                    "name":"old",
                    "vmName":"PageSpace",
                    "collections":1,
                    "used":48269680,
                    "capacity":48660480,
                    "external":836752,
                    "time":0.036075,
                    "avgCollectionPeriodMillis":73634275.99
                }
            }
        }
*
* */

//{time_in_minute:{isolate1:{},isolate2:{}}}

void parseDartVMGCInfo(String gcInfo){
  MyLogger.getDebugLogger()?.trace('dumpDartVMInfos:parseDartVMGCInfo: recv <--- ${gcInfo}');
  try{
    Map gcInfoMap = json.decode(gcInfo) as Map;
    if(!validGCInfo(gcInfoMap)){
      MyLogger.getLogger().error("dumpDartVMInfos recv error gc msg: $gcInfo");
      return ;
    }
    String isolateName = getJsonVal<String>(gcInfoMap,"params/event/isolate/id");
    MyLogger.getDebugLogger()?.trace("dumpDartVMInfos recv isolate gc infos: $isolateName recved yet?? ${currGCInfos.containsKey(isolateName)} ");
    if(currGCInfos.containsKey(isolateName)){
      currGCInfos.remove(isolateName);//更新当前分钟之前的信息
    }
    currGCInfos[isolateName] = gcInfoMap;
  }catch(e){
    MyLogger.getLogger().error("dumpDartVMInfos parseDartVMGCInfo: $e");
  }
}

bool validGCInfo(Map gcInfo){
  if(gcInfo!=null && gcInfo["params"]!=null){
    Map params = gcInfo["params"] as Map;
    return params.containsKey("streamId") && params["streamId"] == "GC" && params.containsKey("event");
  }
  return false;
}

class vmServiceLog extends Log {
  void warning(String message) => MyLogger.getLogger().warn("vmServiceLog $message");
  void severe(String message) => MyLogger.getLogger().error("vmServiceLog $message");
}

class GCInfos{
  int gcCnt ;
  double gcTime;
  int lastMinuteGCCnt;
  double lastMinuteGCTime;
  int used;
  int capacity;
  String memType;

  GCInfos(String type){
    gcCnt = 0;
    gcTime = 0.0;
    used = 0;
    capacity = 0;
    lastMinuteGCTime = 0.0;
    lastMinuteGCCnt = 0;
    memType = type;
  }
}

class GCStatistics{
  GCInfos newSpace;
  GCInfos oldSpace;

  GCStatistics(){
    newSpace = new GCInfos("new");
    oldSpace = new GCInfos("old");
  }
}

Future getCpuUtility(int pid)async {
  if (!Platform.isLinux) {
    return;
  }

  final fcpuStat = new File('/proc/stat');
  final fprocessStat = new File("/proc/$pid/stat");
  // Read the file
  String cpuStat = await fcpuStat.readAsString();
  List<String> cpuStatArr = cpuStat.split(new RegExp(r"\s+\b|\b\s")).sublist(1,11);

  String processStat = fprocessStat.readAsStringSync();
  List<String> processStatArr = processStat.split(new RegExp(r"\s+\b|\b\s")).sublist(13,17);

  int totalCpuTime1 = 0;
  cpuStatArr.forEach((v){totalCpuTime1+=int.tryParse(v)??0;});
  int processCpuTime1 = 0;
  processStatArr.forEach((v){processCpuTime1+=int.tryParse(v)??0;});

  await new Future.delayed(new Duration(milliseconds: 100));

  String cpuStat2 = fcpuStat.readAsStringSync();
  List<String> cpuStatArr2 = cpuStat2.split(new RegExp(r"\s+\b|\b\s")).sublist(1,11);

  String processStat2 = fprocessStat.readAsStringSync();
  List<String> processStatArr2 = processStat2.split(new RegExp(r"\s+\b|\b\s")).sublist(13,17);

  int totalCpuTime2 = 0;
  cpuStatArr2.forEach((v){totalCpuTime2+=int.tryParse(v)??0;});
  int processCpuTime2 = 0;
  processStatArr2.forEach((v){processCpuTime2+=int.tryParse(v)??0;});
  MyLogger.getDebugLogger()?.trace("getCpuUtility $totalCpuTime2-$totalCpuTime1 =${totalCpuTime2-totalCpuTime1} "
      " $processCpuTime2-$processCpuTime1 = ${processCpuTime2-processCpuTime1}");
  double CpuUtility = (processCpuTime2-processCpuTime1)/(totalCpuTime2-totalCpuTime1)*4*100;

  ServiceMonitor serviceMonitor = new ServiceMonitor()
    ..localIP = IPAddress.getHostIp()
    ..mtopPath = "CputStatistic"
    ..rt = CpuUtility.round()
    ..req = "$CpuUtility"
    ..success = true
    ..respSize = 0
    ..logType = LogType.DartVMInfo;

  MyLogger.getLogger(name:"caiNiaoMonitor")?.error("${serviceMonitor.toString()}");
}

