import 'package:saker/framework/log/dart_spdlog_extension.dart';
import 'package:saker/framework/log/log.dart';
import 'package:saker/framework/services/service_flow_limit_base.dart';

class ServiceFlowLimit {
  String servicePath = null;
  int serviceStasticAddr = null;

  String getServicePath() {
    return servicePath;
  }

  void registerStatistic() {
    if (getServicePath() == null) {
      return null;
    }

    serviceStasticAddr = service_statistic_register(
        getServicePath(), new DateTime.now().millisecondsSinceEpoch);
    MyLogger.getLogger().debug("registerStatistic for service:$servicePath serviceStasticAddr=${serviceStasticAddr}");
  }


  int fetchCurrentQps() {
    if (serviceStasticAddr == null) {
      return 0;
    }
    return service_statistic_fetch_qps(
        serviceStasticAddr, new DateTime.now().millisecondsSinceEpoch);
  }

  int incCurrentQps() {
    if (serviceStasticAddr == null) {
      return 0;
    }
    int val = service_statistic_add_and_fetch(
        serviceStasticAddr, 1, new DateTime.now().millisecondsSinceEpoch);
    MyLogger.getDebugLogger()?.debug("${getServicePath()} concurrence cnt: $val");
    return val;
  }

  int getQpsLimit() {
    if (serviceStasticAddr == null || ServiceFlowLimitBase.instance == null) {
      return 0;
    }

    return ServiceFlowLimitBase.instance.getQpsLimit(getServicePath());
  }

  bool flowLimitBlock() {
    MyLogger.getDebugLogger()?.debug("fetchCurrentQps:${fetchCurrentQps()}  getQpsLimit:${getQpsLimit()}");
    if (getQpsLimit() > 0) {
      if (fetchCurrentQps() > getQpsLimit() - 1) {
        return true;
      }
      incCurrentQps();
    }
    return false;
  }
}