import 'package:saker/framework/trace/trace_info.dart';

abstract class SwakTagGroupParser<T> {
  List<String> parse(T var1);
}

enum ReduceType {
  localBreak,
  reduce,
}

abstract class SwakReducer<T> {
  ReduceType reduceType = ReduceType.localBreak;

  T merge(T inputParam, Map<String, T> resultMap);
}

class SwakReducerManager {
  static Map<Type, Map<String, SwakReducer>> _ReducerMap = {};

  static SwakReducer<T> getReducer<T>(Type interfaceType, String methodName) {
    if (!_ReducerMap.containsKey(interfaceType) || !_ReducerMap[interfaceType].containsKey(methodName)) {
      return null;
    }
    dynamic reducer = _ReducerMap[interfaceType][methodName];
    if (reducer is SwakReducer<T>) {
      return reducer;
    }
    return null;
  }

  static void registerReducer(Type interfaceType, String methodName, SwakReducer reducer) {
    if (!_ReducerMap.containsKey(interfaceType)) {
      _ReducerMap[interfaceType] = new Map();
    }

    _ReducerMap[interfaceType][methodName] = reducer;
  }
}

class SwakInfo {
  final SwakTagGroupParser tagGroupParser;
  List<String> swakTags = [];

  SwakInfo(this.tagGroupParser);

  bool match(Set<String> tagMap) {
    if (swakTags == null || swakTags.isEmpty) {
      return false;
    }

    for(String tag in swakTags) {
      if (!tagMap.contains(tag)) {
        return false;
      }
    }
    return true;
  }

  static void buildSwakInfo<T>(SwakTagGroupParser<T> parser, T baseInput) {
    SwakInfo swakInfo = new SwakInfo(parser);
    swakInfo.swakTags = parser.parse(baseInput);

    TraceInfo.setSwakInfo(swakInfo);
  }
}