import 'dart:async';
import 'dart:convert';

import 'package:saker/framework/hsf/hsf_object_base.dart';
import 'package:saker/framework/services/mtop_context.dart';
import 'package:saker/framework/services/hsf_rpc_context_info.dart';
import 'package:saker/framework/stat/service_monitor.dart';
import 'package:saker/framework/swak/swak.dart';

class TraceInfo {
  String eagleeyeTraceid;
  String eagleeyeRpcid;
  int eagleeyeIndex;
  String eagleeyeUserData;
  String ttid;
  String mtopPath;
  StringBuffer debugLogBuffer;
  String pressTestTag;

  Map<String,String> eagleeyeContext;
  String  eagleeyeCtxJson;
  ServiceMonitor serviceMonitor;
  SwakInfo swakInfo;
  //RcpContext
  String rpcContextHessianJson;
  //是否在HotReload的Isolate中
  bool inReloadIsolate = false;
  //给业务保存一个临时的数据
  Map<String, dynamic> bizContext = {};

  static TraceInfo getTraceInfo() {
    return Zone.current[#TraceInfo] as TraceInfo;
  }

  static SwakInfo getSwakInfo() {
    TraceInfo traceInfo = Zone.current[#TraceInfo] as TraceInfo;
    if (traceInfo == null) {
      return null;
    }
    return traceInfo.swakInfo;
  }

  static void setSwakInfo(SwakInfo swakInfo) {
    TraceInfo traceInfo = Zone.current[#TraceInfo] as TraceInfo;
    if (traceInfo == null) {
      return;
    }
    traceInfo.swakInfo = swakInfo;
  }

  static ServiceMonitor getServiceMonitor(){
    TraceInfo traceInfo = Zone.current[#TraceInfo] as TraceInfo;
    if (traceInfo == null) {
      return null;
    }
    return traceInfo.serviceMonitor;
  }
  
  static String getEagleeyeTraceId() {
    TraceInfo traceInfo = Zone.current[#TraceInfo] as TraceInfo;
    if (traceInfo == null) {
      return "";
    }
    return traceInfo.eagleeyeTraceid??"";
  }
  static String getPressTestContext() {
    TraceInfo traceInfo = Zone.current[#TraceInfo] as TraceInfo;
    if (traceInfo == null) {
      return "";
    }
    return traceInfo.pressTestTag??"";
  }
  static String getEagleeyeRpcId() {
    TraceInfo traceInfo = Zone.current[#TraceInfo] as TraceInfo;
    if (traceInfo == null) {
      return "";
    }

    if (traceInfo.eagleeyeRpcid == null || traceInfo.eagleeyeIndex == null) {
      return "0";
    }
    traceInfo.eagleeyeIndex ++;
    return '${traceInfo.eagleeyeRpcid}.${traceInfo.eagleeyeIndex}';
  }

  static String getttid() {
    TraceInfo traceInfo = Zone.current[#TraceInfo] as TraceInfo;
    if (traceInfo == null) {
      return "";
    }
    return traceInfo.ttid??"";
  }

  static String getMtopPath() {
    TraceInfo traceInfo = Zone.current[#TraceInfo] as TraceInfo;
    if (traceInfo == null) {
      return "";
    }
    return traceInfo.mtopPath??"";
  }

  static void setSvcMonitorRespSize(int respSize) {
    TraceInfo traceInfo = Zone.current[#TraceInfo] as TraceInfo;
    if (traceInfo == null) {
      return ;
    }
    traceInfo.serviceMonitor.respSize = respSize;
  }

  static void setSvcMonitorExtraMsg(String extraMsg) {
    TraceInfo traceInfo = Zone.current[#TraceInfo] as TraceInfo;
    if (traceInfo == null) {
      return ;
    }
    traceInfo.serviceMonitor.extraMsg = extraMsg;
  }

  static String getRpcContextHessianJson() {
    TraceInfo traceInfo = Zone.current[#TraceInfo] as TraceInfo;
    if (traceInfo == null) {
      return "";
    }
    if (traceInfo.rpcContextHessianJson != null) {
      return traceInfo.rpcContextHessianJson;
    }

    bool hasData = false;
    DartHsfRpcContext rpcContext = new DartHsfRpcContext();

    MtopContext mtopContext = MtopContext.getZoneContext();
    if (mtopContext != null) {
      hasData = true;
      rpcContext.imei = mtopContext.imei;
      rpcContext.imsi = mtopContext.imsi;
      rpcContext.userId = mtopContext.userId;
      rpcContext.nick = mtopContext.userNick;
      rpcContext.ttid = mtopContext.ttid;
      rpcContext.ip = mtopContext.ip;
      rpcContext.platform = mtopContext.platform;
      rpcContext.appName = mtopContext.appName;
      rpcContext.appVersion = mtopContext.appVersion;
      rpcContext.sid = mtopContext.sid;
      rpcContext.appKey = mtopContext.appkey;
      rpcContext.deviceId = mtopContext.deviceId;
      rpcContext.lat = mtopContext.lat;
      rpcContext.lng = mtopContext.lng;
      rpcContext.uttid = mtopContext.uttid;
      rpcContext.proxyIp =  mtopContext.proxyIP;
      rpcContext.from = mtopContext.from;
      rpcContext.channelId = "";
    }
    if (traceInfo.eagleeyeUserData != null) {
      hasData = true;
      rpcContext.eagleeyeUserData = traceInfo.eagleeyeUserData;
    }

    if (!hasData) {
      traceInfo.rpcContextHessianJson = "";
    } else {
      traceInfo.rpcContextHessianJson = json.encode(makePair(rpcContext?.toHessianJson(), r'com.taobao.idledomain.dto.dataobject.DartHsfRpcContext'));
    }
    return traceInfo.rpcContextHessianJson;
  }

  static String getEagleeyeCtxJson(){
    TraceInfo traceInfo = Zone.current[#TraceInfo] as TraceInfo;
    if (traceInfo == null) {
      return "";
    }
    if (traceInfo.eagleeyeCtxJson != null) {
      return traceInfo.eagleeyeCtxJson;
    }

    if(traceInfo.eagleeyeContext == null){
      return "";
    }

    //traceInfo.eagleeyeCtxHJ = json.encode( makePairMap(traceInfo.eagleeyeContext == null ? null : new Map.fromIterables(traceInfo.eagleeyeContext.keys.map((dynamic e) => e?.toString()).toList(), traceInfo.eagleeyeContext.values.map((e) => makePair(e,14)).toList()),18,14));
    traceInfo.eagleeyeCtxJson = json.encode(traceInfo.eagleeyeContext);
    return traceInfo.eagleeyeCtxJson;
  }
  static bool isInReloadIsolate() {
    TraceInfo traceInfo = Zone.current[#TraceInfo] as TraceInfo;
    if (traceInfo == null) {
      return false;
    }
    return traceInfo.inReloadIsolate;
  }
}