import 'dart:convert';
import 'package:saker/common/utils/type_cast.dart';

enum REQUEST_CAT {
  HTTP,
  HSF,
  TAIR,
  LOCAL,
  OTHER
}

class TrafficTaskInfo extends OtherRequest{

  static const String COLLECT_TASK = "collect";

  TrafficTaskInfo.fromJson(dynamic json){
    _json["invokeCollectRate"]  = json['invokeCollectRate'];
    _json["taskId"]             = json['taskId'];
    _json["taskType"]           = json['taskType'];
  }

  TrafficTaskInfo(){}

  set invokeCollectRate(Map<String, int> value) => _json["invokeCollectRate"] = value;
  set taskId(int value)                         => _json["taskId"]            = value;
  set taskType(String value)                    => _json["taskType"]          = value;
  Map<String, int> get invokeCollectRate        => new Map<String,int>.from(_json["invokeCollectRate"] as Map);
  int              get taskId                   => _json["taskId"]           as int;
  String           get taskType                 => _json["taskType"]         as String;
}


class InvokeParseResult{
  bool        _isReplay = false;
  String      _invokeId;
  String      _traceId;
  REQUEST_CAT _invokeType;
  List        _invokeParams;
  bool        _isDummy  = false;
  var         _currReq;

  static InvokeParseResult dummyResult = new InvokeParseResult().._isDummy=true;

  @override
  String toString() {
    return
      "{"
          "\"isReplay\":$_isReplay,"
          "\"invokeId\":$_invokeId,"
          "\"traceId\":$_traceId,"
          "\"invokeType\":$_invokeType,"
          "\"invokeParams\":$_invokeParams,"
          "\"isDummy\":$_isDummy,"
          "\"currReq\":$_currReq,"
          "}";
  }

  static bool isValidReplayRequest(InvokeParseResult r) =>
      r!=null           &&
          r._traceId !=null && r._traceId.isNotEmpty &&
          r._isReplay!=null && r._isReplay           &&
          r._isDummy !=null && !r._isDummy ;

  dynamic     get currReq       => _currReq;
  List        get invokeParams  => _invokeParams;
  REQUEST_CAT get invokeType    => _invokeType;
  String      get traceId       => _traceId;
  String      get invokeId      => _invokeId;
  bool        get isReplay      => _isReplay;
  bool        get isDummy       => _isDummy;

  set currReq(value)                    => _currReq      = value;
  set isDummy(bool value)               => _isDummy      = value;
  set invokeParams(List value)          => _invokeParams = value;
  set invokeType(REQUEST_CAT value)     => _invokeType   = value;
  set traceId(String value)             => _traceId      = value;
  set invokeId(String value)            => _invokeId     = value;
  set isReplay(bool value)              => _isReplay     = value;
}


class SubInvokeRequest extends OtherRequest{

  String get invokeIdentity        => _json["invokeIdentity"] as String;
  String get extInfo               => _json["extInfo"]        as String;
  String get stacktrace            => _json["stacktrace"]     as String;
  String get traceId               => _json["traceId"]        as String;
  List   get params                => _json["params"]         as List;
  set params(List value)           => _json["params"]          = value;
  set traceId(String value)        => _json["traceId"]         = value;
  set stacktrace(String value)     => _json["stacktrace"]      = value;
  set extInfo(String value)        => _json["extInfo"]         = value;
  set invokeIdentity(String value) => _json["invokeIdentity"]  = value;
}


class SubInvokeResponse extends OtherRequest{

  SubInvokeResponse.fromJson(String jsonString){
    _json = jsonDecode(jsonString) as Map;
  }

  SubInvokeResponse();

  bool   get invokeThrough => _json['invokeThrough'] as bool;
  String get returnObject  => _json['returnObject']  as String;
  bool   get success       => _json['success']       as bool;

  set invokeThrough(bool value)  => _json["invokeThrough"] = value;
  set returnObject(String value) => _json["returnObject"]  = value;
  set success(bool value)        => _json["success"]       = value;
}


class InvokeCollectRequest extends OtherRequest{

  InvokeCollectRequest.fromJson(Map json){
    this._json         = json;
  }

  InvokeCollectRequest(){}

  TrafficTaskInfo               get taskInfo        => _json["taskInfo"] as TrafficTaskInfo;
  List<SubInvokeCollectRequest> get subInvokeList   {
    List l = safeCast(_json["subInvokeList"]);
    List<SubInvokeCollectRequest>  l1 = new List<SubInvokeCollectRequest>();
    l.forEach((e){
      l1.add(new SubInvokeCollectRequest.fromJson(e as Map));
    });
    return l1;
  }
  String                        get traceId         => _json["traceId"]    as String;
  dynamic                       get mainInvoke      => _json["mainInvoke"];

  set taskInfo(TrafficTaskInfo value)                    => _json["taskInfo"]       = value;
  set traceId(String value)                              => _json["traceId"]        = value;
  set subInvokeList(List<SubInvokeCollectRequest> value) => _json["subInvokeList"]  = value;
  set mainInvoke(dynamic value)                          => _json["mainInvoke"]     = value;

  @override
  REQUEST_CAT requestCat() {
    print("************************** ${(this.mainInvoke as BaseJsonRequest).requestCat()}");
   return  (this.mainInvoke as BaseJsonRequest).requestCat();
  }

  @override
  toString()=>"{\"mainInvoke\":${_json["mainInvoke"]},"
      "\"subInvokeList\":${_json["subInvokeList"]},"
      "\"taskInfo\":${_json["taskInfo"]},"
      "\"traceId\":\"${_json["traceId"]}\"}";
}

abstract class BaseJsonRequest{

  Map  _json = {};

  REQUEST_CAT requestCat();

  bool isReplayRequest();

  Map get self =>_json;

  String requestCatString() => requestCat().toString().split('.').last;

  @override
  toString() => jsonEncode(_json);

}


class OtherRequest extends BaseJsonRequest{

  @override
  REQUEST_CAT requestCat() => REQUEST_CAT.OTHER;

  @override
  bool isReplayRequest() => false;


}

class HttpMainInvoke extends BaseJsonRequest{

  @override
  REQUEST_CAT requestCat() => REQUEST_CAT.HTTP;

  @override
  String toString()   => jsonEncode(_json);

  @override
  bool isReplayRequest({key : "isFromDoomReplay"}) {
    if(this.queryString  == null){
      return false;
    }
    return Uri.splitQueryString(queryString)[key] == "true";
  }

  HttpMainInvoke.fromJson(Map json){
    this._json = json;
  }

  HttpMainInvoke(){}

  String get methodType                     => _json["methodType"]  as String;
  String get queryString                    => _json["queryString"] as String ;
  Map<String, List> get requestHeaders      => new Map<String,List>.from(_json["requestHeaders"] as Map);
  Map<String, List> get responseHeadrs      => new Map<String,List>.from(_json["responseHeadrs"] as Map);
  String            get responseBody        => _json["responseBody"]as String;
  String            get url                 => _json["url"]         as String;
  Map               get features            => _json["features"]    as Map;
  Map<String, List> get formData            => new Map<String,List>.from(_json["formData"] as Map);
  int               get httpStatus          => _json["httpStatus"]  as int;
  int               get index               => _json["index"]       as int;

  set index(int value) => _json["index"] = value;
  set methodType(String value)                 =>  _json["methodType"]     = value;
  set queryString(String value)                =>  _json["queryString"]    = value;
  set requestHeaders(Map<String, List> value)  =>  _json["requestHeaders"] = value;
  set responseBody(String value)               =>  _json["responseBody"]   = value;
  set responseHeadrs(Map<String, List> value)  =>  _json["responseHeadrs"] = value;
  set url(String value)                        =>  _json["url"]            = value;
  set httpStatus(int value)                    =>  _json["httpStatus"]     = value;
  set formData(Map<String, List> value)        =>  _json["formData"]       = value;
  set features(Map value)                      =>  _json["features"]       = value;
}


class SubInvokeCollectRequest extends OtherRequest{

  SubInvokeCollectRequest.fromJson(Map m){
    this._json = m;
  }
  SubInvokeCollectRequest(){}
  int          get index            => _json["index"] as int;
  String       get invokeIdentity   => _json["invokeIdentity"] as String;
  List<String> get params           => new List<String>.from(_json["params"] as List);
  String       get returnObject     => _json["returnObject"] as String;
  set index(int value)              => _json["index"]          = value;
  set invokeIdentity(String value)  => _json["invokeIdentity"] = value;
  set params(List<String> value)    => _json["params"]         = value;
  set returnObject(String value)    => _json["returnObject"]   = value;
}
