import 'dart:async';
import 'dart:io';

import 'package:saker/framework/log/log.dart';
import 'package:saker/framework/log/dart_spdlog_extension.dart';
import 'package:saker/framework/trace/trace_info.dart';

import 'package:saker/plugin/faas/common/instances.dart';
import 'package:saker/plugin/faas/container/conf/gaia_config.dart';


class FaasLogger{

  static String _rootLogPath;
  static String _functionLogPath;
  //由Function Framework打印，容器本身的日志，打印函数容器初始化、状态变更、处理出错堆栈等的日志
  //时间|环境标识|function名称|function版本|日志级别|日志内容
  static String _functionContainerLog;
  //由Function Framework打印，函数调用流水日志，记录函数每次请求响应结果日志，无论函数处理失败或者成功都需打印记录
  //时间|主机|环境标识|function名称|function版本|function容器名称|function容器版本|函数触发器类型|eagleyeId|请求来源IP|函数响应结果|函数执行耗时|函数请求报文大小|函数响应报表大小
  static String _functionAccessLog;
  //业务相关日志，由函数开发者调用函数容器封装的logger打印的业务日志
  //时间|环境标识|function名称|function版本|eagleyeId|日志级别|日志内容|异常堆栈（可选）
  static String _functionBizLog;
  //时间|主机|环境标识|function名称|function版本|监控项名称|监控项标签|监控项值
  //由业务代码打印，监控埋点日志，业务通过日志方式实现自定义监控报表
  static String _functionCustomMonitorLog;


  static void init(String rootPath, String logLevel, String projectName, String tag) {
    spdlog_init();
    //替换最后一个
    _rootLogPath = rootPath.replaceAllMapped(new RegExp("\/gaia\$"), (match)=>'')+'/logs';
    _functionLogPath = _rootLogPath+ "/gaia/function";
    _functionContainerLog = _functionLogPath+"/function-container.log";
    _functionAccessLog = _functionLogPath + "/function-access.log";
    _functionBizLog =_functionLogPath + "/function-biz.log";
    _functionCustomMonitorLog = _functionLogPath+ "/function-custom-monitor.log";

    Directory dir = new Directory(_functionLogPath);
    if(!dir.existsSync()){
      dir.createSync(recursive: true);
    }

    /** 鹰眼日志用 Dart vm gc信息*/
    MyLogger.addNewLogger('dartVM', '/home/admin/logs/eagleeye/stat-eagleeye-jvm.log', "warn", LogFileType.rotate_log, rotate_file_size: 64*1024*1024, rotate_file_cnt: 2);

    /** 函数容器日志[faas_container]：函数容器打印初始化、状态变更、处理出错堆栈，业务日志等等日志。*/
    MyLogger.addNewLogger('default', _functionContainerLog, logLevel, LogFileType.rotate_log, rotate_file_size: 64*1024*1024, rotate_file_cnt: 2,
        format:
            (String msg, String levelName){
          TraceInfo traceInfo = Zone.current[#TraceInfo] as TraceInfo;
          return "${GaiaConfig.env}|${functionName}|${functionVersion}|${traceInfo?.eagleeyeTraceid}|${levelName}|${msg}";
        });

    /** 函数日志：函数开发者调用函数容器封装logger打印业务日志。*/
    MyLogger.addNewLogger('faas_biz', _functionBizLog, logLevel, LogFileType.rotate_log, rotate_file_size: 64*1024*1024, rotate_file_cnt: 2,
        format:
            (String msg, String levelName){
          TraceInfo traceInfo = Zone.current[#TraceInfo] as TraceInfo;
          return "${GaiaConfig.env}|${functionName}|${functionVersion}|${traceInfo?.eagleeyeTraceid}|${levelName}|${msg}";
        });

    /** 函数流水日志[faas_access]：记录函数每次请求响应结果日志，无论函数处理失败或者成功都需打印记录。*/
    MyLogger.addNewLogger('functionAccess', _functionAccessLog, "warn", LogFileType.rotate_log, rotate_file_size: 64*1024*1024, rotate_file_cnt: 2,);

    /** 函数自定义监控日志:业务通过日志方式实现自定义监控报表。*/
    MyLogger.addNewLogger('caiNiaoMonitor', _functionCustomMonitorLog, logLevel, LogFileType.rotate_log, rotate_file_size: 64*1024*1024, rotate_file_cnt: 2);

    MyLogger.addNewLogger('flowLimit', '${_functionLogPath}/logs/flowLimit.log', "warn", LogFileType.rotate_log, rotate_file_size: 64*1024*1024, rotate_file_cnt: 2);
    MyLogger.addNewLogger('trafficGuard', '${_functionLogPath}/logs/trafficGuard.log', logLevel, LogFileType.rotate_log, rotate_file_size: 64*1024*1024, rotate_file_cnt: 2);


  }


}

