//初始化Isolate环境
import 'dart:io';

import 'package:saker/framework/environment/env_base.dart';
import 'package:saker/plugin/faas/container/conf/constant.dart';
import 'package:yaml/yaml.dart';

class GaiaConfig {
  /*** Generic Config**/
  //## 容器版本
  static String sidecarVersion = '1.0';
  static String containerVersion = '1.0';

  //##容器间通讯协议,可选：grpc、http24json、http24PB
  static String communicationsProtocol = 'Http2Json';

  //## 容器通讯端口
  static int communicationsPort = 12800;
  //##fucntion容器管理endponit相关端口
  static int fnContainerManagerPort = 12700;
  //##sidecar对外监听端口
  static int sidecarListenrPort = 12500;
  //##sidecar管理endpoint相关端口
  static int sidecarManagePort = 12600;

  //###容器运行环境,ONLINE、PRE、Daily
  static String env = 'Daily';

  //## 容器运行单元，CENTER、UNSZ
  static String unit = 'CENTER';

  //## 容器核数
  static int cpuCores = 4;

  static void init(String root, String appName) {

    //TODO: mock behavior

    if(root.isEmpty || appName.isEmpty){
      throw FormatException('Expected Root And AppName!');
    }

    EnvFramework.rootPath = root;
    EnvFramework.appName = appName;
    EnvFramework.isFaaS = true;

    File localFile = new File('${root}/container/gaia-container.yml');
    if (localFile.existsSync()) {
      String text = localFile.readAsStringSync();
      YamlMap defaults = loadYaml(text) as YamlMap;
      if (defaults.containsKey('sidecarVersion')) {
        sidecarVersion = defaults['sidecarVersion'] as String;
      }
      if (defaults.containsKey('containerVersion')) {
        containerVersion = defaults['containerVersion'] as String;
      }
      if (defaults.containsKey('communicationsProtocol')) {
        communicationsProtocol = defaults['communicationsProtocol'] as String;
        if(communicationsProtocol=='grpc'){
          EnvFramework.serviceServerKind ='grpc';
        }
      }
      if (defaults.containsKey('communicationsPort')) {
        communicationsPort = defaults['communicationsPort'] as int;
        /** 监听通信端口*/
        EnvFramework.httpPort = communicationsPort;
      }
      if (defaults.containsKey('fnContainerManagerPort')) {
        //@fnContainerManagerPort = int.tryParse(defaults['fnContainerManagerPort'])??0;
        fnContainerManagerPort = defaults['fnContainerManagerPort'] as int;
      }
      if (defaults.containsKey('sidecarListenrPort')) {
        //@sidecarListenrPort = int.tryParse(defaults['sidecarListenrPort'])??0;
        sidecarListenrPort = defaults['sidecarListenrPort'] as int;
      }
      if (defaults.containsKey('sidecarManagePort')) {
        //sidecarManagePort = int.tryParse(defaults['sidecarManagePort'])??0;
        sidecarManagePort = defaults['sidecarManagePort'] as int;
      }
      if (defaults.containsKey('env')) {
        env = defaults['env'] as String;
        for(Env e in Env.values){
          if(e.toString().toLowerCase().endsWith('.${env}'.toLowerCase())){
            EnvFramework.env = e.index;
            break;
          }
        }
      }
      if (defaults.containsKey('unit')) {
        unit = defaults['unit'] as String;
      }
      if (defaults.containsKey('cpuCores')) {
        //@cpuCores = int.tryParse(defaults['cpuCores'])??4;
        cpuCores = defaults['cpuCores'] as int;
        EnvFramework.httpConcurrence = cpuCores;
        if (EnvFramework.httpConcurrence <= 0) {
          EnvFramework.httpConcurrence = 1;
        }
      }


    }else{
      throw FormatException('Expected gaia-container.yaml!');
    }

    File appConfigFile = new File('${root}/function/config/config_${env}.yaml');
    if (appConfigFile.existsSync()) {
      String text = appConfigFile.readAsStringSync();
      YamlMap defaults = loadYaml(text) as YamlMap;

      /** | HSF | */
      if (defaults.containsKey('hsfWorkerThreadCnt')) {
        //@EnvFramework.hsfWorkerThreadCnt = int.tryParse(defaults['hsfWorkerThreadCnt']);
        EnvFramework.hsfWorkerThreadCnt = defaults['hsfWorkerThreadCnt'] as int;
      }else{
        EnvFramework.hsfWorkerThreadCnt = 6;
      }

      if (defaults.containsKey('hsfIOThreadCnt')) {
        //@EnvFramework.hsfIOThreadCnt = int.tryParse(defaults['hsfIOThreadCnt']);
        EnvFramework.hsfIOThreadCnt = defaults['hsfIOThreadCnt'] as int;
      }else{
        EnvFramework.hsfIOThreadCnt = 3;
      }

      if (defaults.containsKey('hsfLogLevel')) {
        //@EnvFramework.hsfLogLevel = int.tryParse(defaults['hsfLogLevel']);
        EnvFramework.hsfLogLevel = defaults['hsfLogLevel'] as int;
      }else{
        EnvFramework.hsfLogLevel = 4;
      }

      if (defaults.containsKey('hsfLogPath')) {
        EnvFramework.hsfLogPath = defaults['hsfLogPath'] as String;
      }else{
        EnvFramework.hsfLogPath = '/home/admin/logs/hsfcpp/hsfcpp.log';
      }

      if (defaults.containsKey('hsfConsumeTimeout')) {
        //@EnvFramework.hsfConsumeTimeout = new Duration(milliseconds: int.tryParse(defaults['hsfConsumeTimeout'])??4000);
        EnvFramework.hsfConsumeTimeout = new Duration(milliseconds: defaults['hsfConsumeTimeout'] as int ??4000);
      }else{
        EnvFramework.hsfConsumeTimeout = new Duration(milliseconds: 4000);
      }

      if (defaults.containsKey('hsfSnapshotPath')) {
        EnvFramework.hsfSnapshotPath = defaults['hsfSnapshotPath'] as String;
      }else{
        EnvFramework.hsfSnapshotPath = '/tmp/snapshot';
      }
      /** -- HSF -- */

      /** | TAIR | */
      if (defaults.containsKey('tairLogPath')) {
        EnvFramework.tairLogPath = defaults['tairLogPath'] as String;
      }else{
        EnvFramework.tairLogPath = '/home/admin/logs/tair/tair.log';
      }

      if (defaults.containsKey('tairLogLevel')) {
        EnvFramework.tairLogLevel = defaults['tairLogLevel'] as String;
      }else{
        EnvFramework.tairLogLevel = 'WARN';
      }

      if (defaults.containsKey('tairIOThreadCnt')) {
        //@EnvFramework.tairIOThreadCnt = int.tryParse(defaults['tairIOThreadCnt'])??2;
        EnvFramework.tairIOThreadCnt = defaults['tairIOThreadCnt'] as int ??2;
      }else{
        EnvFramework.tairIOThreadCnt = 4;
      }
      /** -- TAIR -- */

      if (defaults.containsKey('observeAddr')) {
        EnvFramework.observeAddr = defaults['observeAddr'] as String;
      }

      if (defaults.containsKey('observePort')) {
        //@EnvFramework.observePort = int.tryParse(defaults['observePort'])??8000;
        EnvFramework.observePort = defaults['observePort']as int ??8000;
      }

      /** | HTTP | */
      if (defaults.containsKey('httpAddr')) {
        EnvFramework.httpAddr = defaults['httpAddr'] as String;
      }

      if (defaults.containsKey('httpPort')) {
        //@EnvFramework.httpPort = int.tryParse(defaults['httpPort']);
        EnvFramework.httpPort = defaults['httpPort'] as int;
      }

      if (defaults.containsKey('httpConcurrence')) {
        //@EnvFramework.httpConcurrence = int.tryParse(defaults['httpConcurrence'])??1;
        EnvFramework.httpConcurrence = defaults['httpConcurrence'] as int ??1;
        if (EnvFramework.httpConcurrence <= 0) {
          EnvFramework.httpConcurrence = 1;
        }
      }

      if (defaults.containsKey('httpDelay')) {
        //@EnvFramework.httpDelay = new Duration(milliseconds: int.tryParse(defaults['httpDelay']));
        EnvFramework.httpDelay = new Duration(milliseconds: defaults['httpDelay'] as int );
      }
      /** -- HTTP -- */

      /*** */
      if (defaults.containsKey('appName')) {
        EnvFramework.appName = defaults['appName'] as String;
      }

      if (defaults.containsKey('hsfProviderDelay')) {
        //@EnvFramework.hsfProviderDelay = new Duration(milliseconds: int.tryParse(defaults['hsfProviderDelay']));
        EnvFramework.hsfProviderDelay = new Duration(milliseconds: defaults['hsfProviderDelay'] as int);
      }

      if (defaults.containsKey('logServerUrl')) {
        EnvFramework.logServerUrl = defaults['logServerUrl'] as String;
      }

      if (defaults.containsKey('monitorLogFile')) {
        EnvFramework.monitorLogFile = defaults['monitorLogFile'] as String;
      }

      /** | HOTRELOAD | */
      if (defaults.containsKey('envStr')) {
        EnvFramework.envStr = defaults['envStr'] as String;
      }

      if (defaults.containsKey('envCode')) {
        //@EnvFramework.env = int.tryParse(defaults['envCode']);
        EnvFramework.env = defaults['envCode'] as int;
      }

      if (defaults.containsKey('ossConf')) {
        EnvFramework.ossConf = defaults['ossConf'] as String;
      }

      if (defaults.containsKey('ossBucket')) {
        EnvFramework.ossBucket = defaults['ossBucket'] as String;
      }

      /** -- HOTRELOAD -- */
      if (defaults.containsKey('hotReloadInterval')) {
        //@EnvFramework.hotReloadInterval = int.tryParse(defaults['hotReloadInterval'])??0;
        EnvFramework.hotReloadInterval = defaults['hotReloadInterval'] as int;
      }

      if (defaults.containsKey('isPluginActived')) {
        String isPluginUsed = defaults['isPluginActived'] as String;
        EnvFramework.isPluginActived =  isPluginUsed.toLowerCase() == "true" ? true : false;
      }

    }
  }
}