import 'dart:async';

import 'package:saker/framework/jsonserializer/proto.dart';
import 'package:saker/framework/jsonserializer/utils.dart';
import 'package:saker/framework/log/log.dart';
import 'package:saker/framework/services/service_base.dart';
import 'package:saker/framework/services/service_result.dart';
import 'package:saker/framework/tair/dart_tair.dart';
import 'package:saker/plugin/faas/common/event/message_queue_event.dart';
import 'package:saker/plugin/faas/common/instances.dart';
import 'package:saker/plugin/faas/container/conf/gaia_config.dart';

@deprecated
const String METAQ = 'metaq';
@deprecated
const String MTOP = 'mtop';
@deprecated
const String DEFAULT = 'default';

@deprecated
Event buildEvent(MtopContext context, final Map<dynamic, dynamic> data) {
  int eventType = _getCurrentEventType(context: context);
  if (eventTypeMap[METAQ] == eventType) {
    //消息事件
    return MessageQueueEvent()..data = data;
  } else if (eventTypeMap[MTOP] == eventType) {
    return MtopEvent(context)..data = data;
  } else {
    throw Exception('Unknown event type ${eventType}');
  }
}

const Map<String, int> eventTypeMap = {'metaq': 2, 'default': 1, 'mtop': 1};

//trigger name
const String func_evt_metaq = 'metaq';
const String func_evt_mtop = 'mtop';
const String func_evt_type = 'type';

//function handler message
const String func_msg_success = "SUCCESS";
const String func_msg_failed = "FAILED";
const String func_msg_provider_error = "FUNCTION_PROVIDER_ERROR";
const String func_msg_flowlimit_blocked = "FAIL_SYS_FLOWLIMIT";
const String func_msg_service_fail = "FAIL_SERVICE_ERROR";

//function context header
const String func_cxt_flowlimit = 'xfunc-service-flowcontrol';
const String func_cxt_trace_id = 'EagleEye-TraceId';
const String func_cxt_rpc_id = 'EagleEye-RpcId';
const String func_cxt_user_data = 'EagleEye-UserData';
const String func_virtual_path = "/v1/api/invokeFunction";

//function result codes
const Map<String, int> func_msg_inter_map = {
  func_msg_provider_error: 100,
  func_msg_service_fail: 501,
  func_msg_success: 200,
  func_msg_failed: 500,
};

//lifecycle callback function
typedef callback = ServiceResult Function(Event event, Context functionContext);

const Map empty = {};

enum HandlerLifecycleEvt { beforeCall, afterCall, flowBlocked }

int _getCurrentEventType({MtopContext context}) {
  if (Zone.current[#FunctionContext] != null) {
    Context ctx = Zone.current[#FunctionContext] as Context;
    return ctx.functionHandlerContext[func_evt_type] as int;
  }

  final MtopContext _context =
      context ?? Zone.current[#MtopContext] as MtopContext;
  if (_context.context[func_evt_type] != null) {
    return eventTypeMap[_context.context[func_evt_type]];
  }
  return eventTypeMap[DEFAULT];
}

//TODO: refactor function hanlder to function api
abstract class FunctionService extends ServiceBase {
  final Map<HandlerLifecycleEvt, callback> _callbacks = {
    HandlerLifecycleEvt.beforeCall: null,
    HandlerLifecycleEvt.afterCall: null,
    HandlerLifecycleEvt.flowBlocked: null
  };

  Map<HandlerLifecycleEvt, callback> get callbacks => _callbacks;

  @override
  Future<ServiceResult> process(ApiRequestBase req, MtopContext context) => null;

  @override
  @deprecated
  Future<ServiceResult> processMtop(
      final Map<dynamic, dynamic> data, final MtopContext mtopContext) {
    Context context =
        Context.request(mtopContext.headers['eagleeye-traceid'] ?? "");
    Future<ServiceResult> result =
        handle(buildEvent(mtopContext, data), context);
    return result;
  }

  @deprecated
  ServiceResult buildMapFailedResult({Map data = empty}) {
    ServiceResult r = ServiceResult()
      ..errcode = 'FAILED'
      ..errmsg = '调用失败';

    if (_getCurrentEventType() == eventTypeMap[METAQ]) {
      data = {'consumeStatus': 'RECONSUME_LATER', 'delayLevelWhenNextCons': 1};
    }
    r.data = JsonUtil.translateMTOP(data) as Map;
    r.data['serverTime'] = getServerTime();
    return r;
  }

  ServiceResult buildMapSuccResult({Map data= empty}) {
    ServiceResult r = ServiceResult()
      ..errcode = 'SUCCESS'
      ..errmsg = '接口调用成功';

    if (_getCurrentEventType() == eventTypeMap[METAQ] && data.isEmpty) {
      data = {'consumeStatus': 'CONSUME_SUCCESS', 'delayLevelWhenNextCons': 1};
    }
    r.data = JsonUtil.translateMTOP(data) as Map;
    r.data['serverTime'] = getServerTime();
    return r;
  }

  @override
  String getServicePath() => "${func_virtual_path}_${functionName}";

  Future<ServiceResult> handle(final Event event, final Context context);
}

class Context {
  String requestId;

  FunctionInfo functionInfo;

  FunctionContainer functionContainerInfo;

  Map<String, dynamic> functionHandlerContext = {};

  Context(this.requestId, this.functionInfo, this.functionContainerInfo);

  Context.request(String requestId)
      : this(
            requestId,
            FunctionInfo()
              ..functionName = functionName
              ..functionVersion = functionVersion,
            FunctionContainer()
              ..runtimeEnv = GaiaConfig.env
              ..runtimeUnit = GaiaConfig.unit
              ..containerVersion = GaiaConfig.containerVersion);

  TairService tairFactory(String tairName) {
    return tairServices[tairName];
  }

  Logger get loggerFactory => MyLogger.getLogger(name: 'faas_biz');
}

class Event {
  final int _eventType;
  Map _data;
  Event(this._eventType);
  int get eventType => _eventType;
  Map get data => _data;
  set data(Map value) {
    _data = value;
  }
}

class MtopEvent extends Event {
  final MtopContext _context;
  MtopEvent(this._context) : super(eventTypeMap[MTOP]);
  MtopContext get mtopContext => _context;
}

// Concepts from Gaia PlatForm
class FunctionInfo {
  //函数名称
  String functionName;

  //函数别名
  String functionAlias;

  //函数版本
  String functionVersion;

  //函数资源配置
  Map<String, dynamic> functionResConfig;

  //函数环境变量
  Map<String, dynamic> functionEnvConfig;
}

class FunctionContainer {
  //容器运行环境
  String runtimeEnv;

  //容器运行单元

  String runtimeUnit;

  //容器版本
  String containerVersion;

  //容器相关信息
  Map<String, dynamic> containerConfigs;
}

