import 'dart:convert';

import 'package:saker/common/utils/type_cast.dart';
import 'package:saker/framework/services/grpc/src/generated/df_gaia_message.pb.dart';
import 'package:saker/framework/services/grpc/src/generated/df_gaia_trigger_metaq.pb.dart';
import 'package:saker/framework/services/grpc/src/generated/google/protobuf/any.pb.dart';
import 'package:saker/plugin/faas/common/event/message.dart';
import 'package:saker/plugin/faas/common/event/message_queue.dart';
import 'package:saker/plugin/faas/common/event/message_queue_event.dart';
import 'package:saker/plugin/faas/container/function.dart';
import 'package:saker/plugin/faas/container/src/df_platform_protocol.dart';

//metaQ impl
EventProccessor<MetaqFunctionRequest, MetaqFunctionResponse, EventResult>
    handleMetaq = (final MetaqFunctionRequest request,
        final FunctionService handler, final Context context) async {
  //init evt
  List<MetaqFunctionRequest_MetaqMessage> messages = request.metaqMsgList;
  PMessageQueue pMessageQueue = PMessageQueue()
    ..brokerName = request.brokerName
    ..metaqMsgList = <PMessage>[];
  messages?.forEach((msg) {
    //byte msg
    pMessageQueue.metaqMsgList.add(PMessage()
      ..msgBody = utf8.decode(msg.msgBody)
      ..messageId = msg.msgId
      ..properties = msg.properties
      ..bornTimestamp = msg.bornTimestamp?.toInt() ?? 0
      ..bodyCRC = msg.bodyCRC
      ..queueId = int.tryParse(msg.queueId)
      ..queueOffset = msg.queueOffset?.toInt() ?? 0
      ..topicName = msg.topicName
      ..tags = msg.tags);
  });

  context.functionHandlerContext['brokerName'] = request.brokerName ?? "";

  return $_handle(request, MetaqFunctionResponse(), handler, context,
      MessageQueueEvent.forMessage(pMessageQueue), _buildMetaQ);
};

//metaQ evnet build
EvtResultBuilder<MetaqFunctionResponse, EventResult> _buildMetaQ =
    (MetaqFunctionResponse response, Map<String, dynamic> handlerContext,
        bool isSucceed,
        {String errorCode = func_msg_success,
        String errorMsg = func_msg_success,
        Map data,
        String buf}) {
  final EventResult eventResult = EventResult()
    ..success = isSucceed
    ..resultMsg = isSucceed ? errorMsg : "请稍候再试"
    ..resultCode = func_msg_inter_map[errorCode].toString();

  if (data != null) {
    response
      ..consumeStatus = safeCast(data['consumeStatus'])
      ..delayLevelWhenNextConsume =
          safeCast(data['delayLevelWhenNextCons']) ?? 1;
    return eventResult..payload = Any.pack(response);
  }

  return eventResult
    ..payload = Any.pack(response
      ..consumeStatus = (isSucceed ? 'CONSUME_SUCCESS' : 'RECONSUME_LATER')
      ..delayLevelWhenNextConsume = 1);
};
