import 'dart:convert';

import 'package:saker/common/utils/type_cast.dart';
import 'package:saker/framework/services/grpc/src/generated/df_gaia_trigger_mtop.pb.dart';
import 'package:saker/framework/services/grpc/src/generated/google/protobuf/any.pb.dart';
import 'package:saker/framework/services/mtop_context.dart';
import 'package:saker/plugin/faas/container/function.dart';
import 'package:saker/plugin/faas/container/src/df_platform_protocol.dart';
import 'package:saker/framework/services/grpc/src/generated/df_gaia_message.pb.dart';

EventProccessor<MtopFunctionRequest, MtopFunctionResponse, EventResult>
    handleMtop = (final MtopFunctionRequest request,
        final FunctionService handler, final Context context) async {
  final MtopFunctionResponse response = MtopFunctionResponse()
    ..contentType = 'application/json'
    ..httpStatusCode = 200;

  final MtopEvent event = MtopEvent(MtopContext({
    'userId': request.userInfo['userId'],
    'nick': request.userInfo['nick'],
    'platform': request.platform,
    'deviceId': request.deviceInfo['deviceId'],
    'ttid': request.ttid,
    'appkey': request.appkey,
    'utdid': request.deviceInfo['utdid'],
    'uttid': request.deviceInfo['uttid'],
    'imei': request.deviceInfo['imei'],
    'imsi': request.deviceInfo['imsi'],
    'appVersion': request.appVersion,
    'lat': request.lat,
    'lng': request.lng,
    'ip': request.remoteIP,
    'appName': request.appName,
    'sid': request.sid
  }, request.headers))
    ..data = safeCast(jsonDecode(utf8.decode(request.data)));

  //overwrite function handler context
  context.functionHandlerContext['api'] = request.api ?? "";
  context.functionHandlerContext['v'] = request.v ?? "";
  context.functionHandlerContext['ttid'] = request.ttid ?? "";
  context.functionHandlerContext['x-remote-ip'] = request.remoteIP ?? "";
  context.functionHandlerContext['data'] = event.data ?? {};
  context.functionHandlerContext['userId'] = request.userInfo['userId'] ?? "";
  context.functionHandlerContext['nick'] =
      request.userInfo['nick'] ?? "";
  return $_handle(request, response, handler, context, event, _buildMtop);
};

EvtResultBuilder<MtopFunctionResponse, EventResult> _buildMtop =
    (MtopFunctionResponse response, Map<String, dynamic> handlerContext,
        bool isSucceed,
        {String errorCode = func_msg_success,
        String errorMsg = func_msg_success,
        Map data,
        String buf}) {
  final EventResult eventResult = EventResult()
    ..success = isSucceed
    ..resultMsg = isSucceed ? errorMsg : "请稍候再试"
    ..resultCode = func_msg_inter_map[errorCode].toString();

  final Map pattern = {
    'api': handlerContext['api'],
    'v': handlerContext['v'],
    'ret': ['${errorCode}::${errorMsg}'],
    'data': data ?? {}
  };

  return eventResult
    ..payload = Any.pack(response
      ..httpStatusCode = 200
      ..data = utf8.encode(json.encode(pattern..['data'] = data)));
};

