import 'package:grpc/src/server/call.dart';
import 'package:saker/framework/event/event_pool.dart';
import 'package:saker/framework/event/event_pool_base.dart';
import 'package:saker/framework/event/notice_type.dart';
import 'package:saker/framework/log/log.dart';
import 'package:saker/framework/reloader/reload_source.dart';
import 'package:saker/framework/services/service_var.dart';
import 'package:saker/plugin/faas/ops/src/generated/df_ops.pbgrpc.dart';

class GrpcOpsService extends OpsServiceBase {
  @override
  Future<OpsResult> hotReload(
      ServiceCall call, HotReloadRequest request) async {
     final Set<String> isolateSet =
        (request.isolateList!=null && request.isolateList.isNotEmpty)
            ? Set<String>.from(request.isolateList.split(',')): Set<String>();
    final String sourceUri = (request.sourceUri?.isEmpty)
        ? '/home/admin/gaia/target/function.dill'
        : request.sourceUri;

    OpsResult result = OpsResult();

    List<String> ids = await findAllHttpIsolateIds();
    bool isOK = true;
    for (String id in ids) {
      if (isolateSet.isEmpty) {
        isOK &= await reloadSource(id, sourceUri);
        continue;
      }
      if (isolateSet.contains(id)) {
        isOK &= await reloadSource(id, sourceUri);
      }
    }
    return result
      ..success = isOK
      ..errorCode = isOK ? 'SUCCESS' : 'FAILED'
      ..errorMsg = isOK ? 'SUCCESS' : 'FAILED';
  }

  @override
  Future<OpsResult> health(ServiceCall call, healthRequest request) async {
    final OpsResult opsResult = OpsResult();
    if (svcOfflineStatus == 0) {
      return opsResult
        ..success = true
        ..errorCode = 'ECHO'
        ..errorMsg = 'ECHO';
    } else {
      MyLogger.getLogger().error('HTTP Health Offline');
      return opsResult
        ..success = false
        ..errorCode = 'OFFLINE'
        ..errorMsg = 'OFFLINE';
    }
  }

  @override
  Future<OpsResult> offline(ServiceCall call, OfflineRequest request) async =>
      _offline(request.offline);

  @override
  Future<OpsResult> online(ServiceCall call, OnlineRequest request) async =>
      _offline(0);

  OpsResult _offline(int offline) {
    final OpsResult result = OpsResult();
    try {
      int nt = offline == 1 ? NoticeType.HttpOffline : NoticeType.HttpOnline;
      mainEventPort.send([
        EventType_Notice,
        [nt, EventPool.sendPort, 0, offline]
      ]);
      result
        ..success = true
        ..errorMsg = 'OK'
        ..errorCode = 'OK';
    } catch (e) {
      result
        ..success = false
        ..errorMsg = 'FAILED'
        ..errorCode = 'FAILED';
    }
    return result;
  }
}
