@TestOn("linux")

import 'dart:async';
import 'dart:convert';
import 'package:saker/framework/environment/env_base.dart';
import 'package:saker/framework/event/event_pool.dart';
import 'package:saker/framework/hsf/hsf_async.dart';
import 'package:saker/framework/hsf/hsf_info.dart';
import 'package:saker/framework/hsf/hsf_config_param.dart';
import 'package:saker/framework/hsf/hsf_rpc_base.dart';
import 'package:test/test.dart';
import 'package:saker/framework/hsf/hsf_object_base.dart';
import 'test_base.dart';

EnvBase _envBase;

void main() {
  setUp(() async {
    TestBase.initConfFile();

    _envBase = new EnvTest(null);
    _envBase.init(DART_TEST_ROOT, 'TEST');

    EnvBase.getEnv =  () => _envBase;

    hsf_core_init(EnvFramework.hsfSnapshotPath, EnvFramework.hsfLogPath,
        EnvFramework.hsfLogLevel, EnvFramework.appName, EnvFramework.hsfIOThreadCnt, EnvFramework.hsfWorkerThreadCnt);
    register_isolate(EventPool.sendPort, null);
  });

  tearDown(() async {
    hsf_core_destroy();
  });

  group('HSF调用测试', () {
    test('调用C接口', () async {
      AsyncTestService.initHsfConsume(EnvFramework.env, new HsfConfigParam());

      int a = await AsyncTestService.getInstance().hsf_square_ff01(1);

      expect(a, 1);
    });
  });
}

class AsyncTestService extends HsfRPCBase {
  static AsyncTestService _asyncServiceService = null;

  static AsyncTestService getInstance() => _asyncServiceService;

  static void initHsfConsume(int env, HsfConfigParam config) {
    AsyncTestService _instance = new AsyncTestService();

    _instance.hsfInfo = new HsfInfo()
      ..aliasName='AsyncService'
      ..consumerName='com.alibaba.idle.hsf.AsyncService'
      ..version='1.0.0.DAILY'
      ..group='HSF';

    _instance.initHsfConsumeBase(config);

    _asyncServiceService = _instance;
  }

  Future<int> hsf_square_ff01(int arg0, {Duration hsfTimeOut:defaultHsfTimeOut, String invokeIp:''}) async {
    List paramsList = [];
    paramsList.add(makePairParameter(arg0,8));

    Map<String, dynamic> paramsMap = {'t': 15, 'v': paramsList};
    String params = json.encode(paramsMap);

    String r = null;
    r = await invoke('hsf_square', 'AsyncTestService.hsf_square_ff01', params, null, null, hsfTimeOut, invokeIp, hsfInvokeTypeNative);

    if (r == null) {
      return null;
    }

    Map<String, dynamic> resultMap = (json.decode(r) as Map).cast();
    int data = resultMap['Res'] as int;
    return data;
  }
}
