import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:saker/framework/environment/env_base.dart';
import 'package:saker/framework/log/log.dart';
import 'package:saker/framework/services/service_base.dart';

int TEST_HTTP_PORT = 8080;
String DART_TEST_ROOT = '/tmp/dart-test';

overridePrint(testFn()) => () async {
  await runZoned(() async {
    await testFn();
  }, zoneSpecification: new ZoneSpecification(
      print: (self, parent, zone, message) {
        // Ignore message.
      }));
};

class EnvTest extends EnvBase {
  Map<String, ServiceBase> serviceEntries;

  EnvTest(this.serviceEntries);

  @override
  Map<String, ServiceBase> getServiceEntries() {
    return serviceEntries;
  }

  @override
  void init(String root, String tag) {
    EnvFramework.init(root);
    MyLogger.initLogger(EnvFramework.rootPath, 'critical', EnvFramework.appName, tag);
  }

  @override
  void initApp() {
  }

  @override
  void initIsolate() {
  }

  @override
  void releaseIsolate() {
  }

  @override
  void closeApp() {
  }
}

class TestBase {
  static File getFrameworkSrcConfFile() {
    print(Directory.current.path);
    return new File('${Directory.current.path}/test/conf/framework.conf');
  }

  static void initConfFile() {
    Directory dir = new Directory('${DART_TEST_ROOT}/logs');
    if (!dir.existsSync()) {
      dir.createSync(recursive: true);
    }

    if (new Directory('/home/admin').existsSync() && !new Directory('/home/admin/logs').existsSync()) {
      new Directory('/home/admin/logs').createSync(recursive: true);
    }

    new File('${DART_TEST_ROOT}/framework.conf').writeAsBytesSync(TestBase.getFrameworkSrcConfFile().readAsBytesSync());
  }

  static String mapToQuery(Map<String, String> map, {Encoding encoding}) {
    var pairs = <List<String>>[];
    map.forEach((key, value) =>
        pairs.add([
          Uri.encodeQueryComponent(key, encoding: encoding),
          Uri.encodeQueryComponent(value, encoding: encoding)
        ]));
    return pairs.map((pair) => "${pair[0]}=${pair[1]}").join("&");
  }
}
